#include "CMcAdptManager.h"
#include "../socket/TcpServer.h"
#include "../base_module/factory.h"
#include "jt808/CJT808Terminal.h"
#include "config/CConfigManager.h"
#include "timer/CTimerManager.h"

REGISTER_MODULE(CMcAdptManager, "CMcAdptManager")

const QString MIC_DB_NAME = "macrophone";

CMcAdptManager::CMcAdptManager(QObject *parent)
    : QObject(parent),
      IModule(),
      m_pTcpModule(nullptr),
      m_curAudioStatus(0),
      m_pJt808Terminal(nullptr),
      m_pDbMic(nullptr),
      m_pConfigMng(nullptr),
      m_iStatusRespCnt(0)
{
}

CMcAdptManager::~CMcAdptManager()
{
    std::cout << name() << std::endl;
}

//初始化操作
bool CMcAdptManager::Initialize()
{
    InitializeDB();
    //起主消息处理线程
    STARTUP_TASK(TaskMain)

    return true;
}

// 初始化数据库
bool CMcAdptManager::InitializeDB()
{
    // 注册数据库
    m_pConfigMng = dynamic_cast<CConfigManager *>(QueryPlugin("CConfigManager"));
    if (m_pConfigMng)
    {
        m_pDbMic = m_pConfigMng->getDatabase(MODULE_MC_ADPT, MIC_DB_NAME);
        if (!m_pDbMic)
        {
            m_pDbMic = m_pConfigMng->regDatabase(MODULE_MC_ADPT, MIC_DB_NAME);
            m_pDbMic->regKey("mic_config", "");
            m_pDbMic->save();
        }
    }

    return true;
}

// 保存数据库
bool CMcAdptManager::saveConfig()
{
    if (!m_pDbMic)
    {
        m_pDbMic->save();
        return true;
    }
    return false;
}

bool CMcAdptManager::RegistMessage()
{
    m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    if (m_pTcpModule)
    {
        m_pTcpModule->RegistMessageCallback(this, MODULE_MC_ADPT);
    }

    // 定时器模块
    CTimerManager *pTimerMng = dynamic_cast<CTimerManager *>(::QueryPlugin("CTimerManager"));
    if (pTimerMng)
    {
        pTimerMng->RegTimer(MODULE_MC_ADPT, TIMER_GET_MC_STATUS, 10);
    }

    return true;
}

//主消息任务，主要用于和其他模块之间的消息通讯
void CMcAdptManager::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskAudioAdpt");
    MODULE_MESSAGE *pMsg = nullptr;
    RegistMessage();
    waitForStart();
    regJt808CmdProc();

    emit notifyAudioStatus(m_curAudioStatus);

    while (!isShutdown())
    {
        if (GetModuleMessage(pMsg)) // get msg from message queue
        {
            switch (pMsg->iCmd)
            {
            case CMD_MC_GET_CONFIG:
                ProcGetAudioConfig(*pMsg);
                break;
            case CMD_MC_REP_STATUS:
                ProcRepAudioStatus(*pMsg);
                break;
            case TIMER_GET_MC_STATUS:
                ProcGetMcStatus(*pMsg);
                break;
            default:
                break;
            }

            SAFE_DELETE(pMsg);
        }
    }
}

// 处理获取配置
void CMcAdptManager::regJt808CmdProc()
{
#define CALLBACK_TEMPLATE(F) (std::bind(&CMcAdptManager::F, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3))

    JT808_MSG_PROC_NODE reg_array[] = {
        {SERVER_SET_MACROPHONE_CONFIG, CALLBACK_TEMPLATE(procJt808SetMcConfig)},
    };

    m_pJt808Terminal = dynamic_cast<CJT808Terminal *>(QueryPlugin("CJT808Terminal"));
    if (m_pJt808Terminal)
    {
        m_pJt808Terminal->registJt808Messages(reg_array, sizeof(reg_array) / sizeof(JT808_MSG_PROC_NODE));
    }
}

/*******************************************
函数功能: 下发麦克风配置
入口参数: cycle_num 流水号
         payload 数据包
         lenth 长度
出口参数:
备    注:
*******************************************/
void CMcAdptManager::procJt808SetMcConfig(int cycle_num, BYTE *payload, WORD lenth)
{
    qDebug() << "<<--下发麦克风配置";

    QString config = QString("%1").arg((char *)payload, lenth);
    // 保存配置信息
    if (m_pDbMic)
    {
        m_pDbMic->setValue("mic_config", config);
        saveConfig();
    }
    // 下发配置到应用层
    sendGetAudioConfig(config);

    qDebug() << "-->>下发麦克风配置应答";
    if (m_pJt808Terminal)
    {
        m_pJt808Terminal->sendCommonAnswer(cycle_num, SERVER_SET_MACROPHONE_CONFIG, 0);
    }
}

// 处理获取配置
void CMcAdptManager::ProcGetAudioConfig(MODULE_MESSAGE &recv_msg)
{
    // 读取配置信息
    QString config = "";
    // 保存配置信息
    if (m_pDbMic)
    {
        config = m_pDbMic->getValue("mic_config").toString();
    }
    // 下发配置到应用层
    sendGetAudioConfig(config);
}

// 下发配置到应用层
void CMcAdptManager::sendGetAudioConfig(QString config)
{
    //给应用层发送消息,同步传感器信息
    MODULE_MESSAGE msg;
    msg.dst_id = MODULE_MC_MOD;
    msg.dst_type = APP_TYPE_MICROPHONE;
    msg.iCmd = CMD_MC_SET_CONFIG;
    strncpy((char *)msg.szBuff, config.toStdString().c_str(), MAX_MESSAGE_BUFF_SIZE);

    //模块间发送消息
    m_pTcpModule->SendMsg(msg, sizeof(msg));
}

// 处理上报麦克风状态
void CMcAdptManager::ProcRepAudioStatus(MODULE_MESSAGE &msg)
{
    memcpy(&m_curAudioStatus, msg.szBuff, sizeof(int));

    emit notifyAudioStatus(m_curAudioStatus);
    m_curAudioStatus = 0;
}

// 获取麦克风状态
void CMcAdptManager::ProcGetMcStatus(MODULE_MESSAGE &recv_msg)
{
    if (m_iStatusRespCnt > 3)
    {
        m_curAudioStatus = 0;
        emit notifyAudioStatus(m_curAudioStatus);
    }
    //给应用层发送消息,同步传感器信息
    MODULE_MESSAGE msg;
    msg.dst_id = MODULE_MC_MOD;
    msg.dst_type = APP_TYPE_MICROPHONE;
    msg.iCmd = CMD_MC_GET_STATUS;

    //模块间发送消息
    m_pTcpModule->SendMsg(msg, sizeof(msg));
    m_iStatusRespCnt++;
}
