﻿#include <string>
#include "base_module.h"
#include <mutex>
#include <map>
#include "factory.h"
#include <QDebug>
#include <QDateTime>
#include "message.h"
#include <assert.h>

// 模块map表
MODULE_MAP IModule::s_ObjList;

MODULE_QUEUE IModule::s_ModuleQueue;

// 获取模块map表
MODULE_MAP &IModule::getModuleMap()
{
    return s_ObjList;
}

// 获取模块栈
MODULE_QUEUE &IModule::getModuleQueue()
{
    return s_ModuleQueue;
}

// 向模块推送消息
bool IModule::PostModuleMessage(MODULE_MESSAGE *msg)
{
    bool bSuccess = false;
    QDateTime date = QDateTime::currentDateTime();
    if (m_MsgQue.size() < MAX_QUEUE_LEN)
    {
        m_msgMtx.lock();
        //qDebug()<<"[PostModuleMessage]"<<name().c_str()<<"模块:"<<(int)msg->dst_id<<"消息字:"<<(int)msg->iCmd;
        m_MsgQue.push_back(msg);
        m_msgMtx.unlock();
        m_msgSem.release();
        bSuccess = true;
    }
    return bSuccess;
}

// 获取消息
bool IModule::GetModuleMessage(MODULE_MESSAGE *&pMsg)
{
    bool bSuccess = false;
    m_msgSem.acquire();
    if (m_MsgQue.size() > 0)
    {
        m_msgMtx.lock();
        pMsg = m_MsgQue.front();
        //qDebug() << "[GetModuleMessage]" << name().c_str() << "模块:" << (int)pMsg->dst_id << "消息字:" << (int)pMsg->iCmd;
        m_MsgQue.pop_front();
        m_msgMtx.unlock();
        bSuccess = true;
    }
    else
    {
        //qDebug() << "================GetModuleMessage ERROR[m_MsgQue.size() == 0]====================";
        pMsg = nullptr;
    }
    return bSuccess;
}

// 起线程
pthread_t IModule::startupTask(PTHREAD_FUNC task, void *pThis)
{
    pthread_t thread_id;
#ifdef WIN32
    //起线程
    _beginthreadex(nullptr, 0, (_beginthreadex_proc_type)task, pThis, 0, &thread_id);
#else
    //起线程
    pthread_create(&thread_id, nullptr, task, pThis);
#endif
    return thread_id;
}

// 模块是否安全
bool IModule::isSafeModule()
{
    try
    {
        if (begin_flag == VALID_BEGIN_FLAG && end_flag == VALID_END_FLAG)
        {
            return true;
        }
    }
    catch (...)
    {
        return false;
    }

    return false;
}

//主任务开启
void IModule::start()
{
    m_startSem.release();
}

//等待主任务开启
void IModule::waitForStart()
{
    m_startSem.acquire();
}

// 安全关闭模块
void IModule::shutdown()
{
    m_bShutdown = true;
    MODULE_MESSAGE *pMsg = new MODULE_MESSAGE();
    pMsg->iCmd = CMD_APP_QUIT;
    PostModuleMessage(pMsg);
}

//模块初始化
extern bool LoadPlugin(string strModule)
{
    IModule *module = QueryPlugin(strModule);

    // 没有找到时，产生新的模块加入map
    if (!module)
    {
        /* code */
        module = factory<IModule>::produce(strModule);
        if (module)
        {
            IModule::getModuleMap()[strModule] = module;
            IModule::getModuleQueue().push_back(module);
        }
        else
        {
            std::cout << "不存在模块:" << strModule << std::endl;
            return false;
        }
    }

    // 初始化
    if (!module->Initialize())
    {
        std::cout << "模块初始化失败:" << strModule << std::endl;
        return false;
    }

    return true;
}

//模块查询
extern IModule *QueryPlugin(string strModule)
{
    //std::cout << "查询模块实例: " << strModule<<std::endl;
    if (IModule::getModuleMap()[strModule])
    {
        //std::cout << "找到模块实例: " << strModule<<std::endl;
        return (IModule::getModuleMap()[strModule]);
    }
    else
    {
        return nullptr;
    }
}

// 启动全部模块的主任务(模块加载后主任务处于挂起等待状态)
extern void startUpAll()
{
    IModule *module = nullptr;
     auto it = IModule::getModuleQueue().begin();
     for (; it != IModule::getModuleQueue().end(); it++)
     {
         module = dynamic_cast<IModule *>(*it);
         if (module && module->isSafeModule())
         {
             module->start();
         }
     }

    // MODULE_MAP::iterator it = IModule::getModuleMap().begin();
    // for (; it != IModule::getModuleMap().end(); it++)
    // {
    //     module = dynamic_cast<IModule *>(it->second);
    //     if (module && module->isSafeModule())
    //     {
    //         module->start();
    //     }
    // }
}

// 安全关闭全部任务
extern void ShutdownAll()
{
    IModule *module = nullptr;
     auto it = IModule::getModuleQueue().rbegin();
     for (; it != IModule::getModuleQueue().rend(); it++)
     {
         module = dynamic_cast<IModule *>(*it);
         if (module && module->isSafeModule())
         {
             module->shutdown();
         }
     }

//    MODULE_MAP::iterator it = IModule::getModuleMap().begin();
//    for (; it != IModule::getModuleMap().end(); it++)
//    {
//        module = dynamic_cast<IModule *>(it->second);
//        if (module && module->isSafeModule())
//        {
//            module->shutdown();
//        }
//    }

    // 设置一个等待时间，让所有任务都能够主动退出
    mSleep(5000);

    it = IModule::getModuleQueue().rbegin();
    for (; it != IModule::getModuleQueue().rend(); it++)
    {
        module = dynamic_cast<IModule *>(*it);
        if (module && module->isSafeModule())
        {
            SAFE_DELETE(module)
        }
        else
        {
            assert(0);
        }
    }

    // 给gstreamer一段时间作析构
    mSleep(3000);
}

//设置运行等级SCHED_RR, SCHED_FIFO, SCHED_OTHER
void setTaskRunType(int sched)
{
    (void)sched;
#ifndef WIN32
    pthread_attr_t attr;
    int policy = 0;
    pthread_attr_init(&attr);
    pthread_attr_setschedpolicy(&attr, sched); //
    pthread_attr_getschedpolicy(&attr, &policy);

    if (policy == SCHED_FIFO)
    {
        cout << "Schedpolicy:SCHED_FIFO\n";
    }
    else if (policy == SCHED_RR)
    {
        cout << "Schedpolicy:SCHED_RR\n";
    }
    else if (policy == SCHED_OTHER)
    {
        cout << "Schedpolicy:SCHED_OTHER\n";
    }
#endif
}

// 设置任务名称
void setCurrentTaskName(string thread_name)
{
#ifdef WIN32
    (void)thread_name;
#else
    prctl(PR_SET_NAME, thread_name.c_str());
#endif
}
