﻿#pragma once
#include <iostream>
#include <string>
#include <thread>
#include <map>
#include <vector>
#include <list>
#include <mutex>
#include <QSemaphore>
#include "common.h"

#ifdef WIN32
#include <process.h>
typedef unsigned int pthread_t;
#else
#include <sys/prctl.h>
#endif

typedef void *(*PTHREAD_FUNC)(void *);
using namespace std;

// 线程启动定义，这个封装让任务函数在类里面运行
#define THREAD_DEFINITION(class_name, _MainTask)   \
    static void *begin_##_MainTask##_(void *pPara) \
    {                                              \
        class_name *p = (class_name *)(pPara);     \
        p->_MainTask(pPara);                       \
        return nullptr;                            \
    }

//启动任务
#define STARTUP_TASK(_MainTask) \
    startupTask(&begin_##_MainTask##_, this);

// 有效性标记位，野指针规避操作
#define VALID_BEGIN_FLAG 0x55AA
#define VALID_END_FLAG 0xAA55
#define INVALID_FLAG 0xFFFF

// 最大队列长度，考虑到现有的消息频度，512队列长度可以满足日常使用
#define MAX_QUEUE_LEN 512

// 模块消息
struct MODULE_MESSAGE;

class IModule;

// 模块map表
typedef std::map<string, IModule *> MODULE_MAP;

// 模块map表
typedef std::vector<IModule *> MODULE_QUEUE;

// 模块基类
class IModule
{
public:
    // 构造函数
    IModule() : begin_flag(VALID_BEGIN_FLAG),
                m_msgSem(0),
                m_startSem(0),
                m_nMainThreadID(0),
                m_bShutdown(false),
                end_flag(VALID_END_FLAG)
    {
    }

    // 析构函数
    virtual ~IModule()
    {
        m_bShutdown = true;
        end_flag = INVALID_FLAG;
        begin_flag = INVALID_FLAG;
    }

    //模块名称
    virtual string name() = 0;

    //初始化操作
    virtual bool Initialize() = 0;

    // 模块是否安全
    bool isSafeModule();

    // 向模块推送消息
    bool PostModuleMessage(MODULE_MESSAGE *msg);

    // 调用start 开时统一启动，防止模块间存在依存的情况下
    virtual void start();

    // 关闭模块，主要功能是安全删除启动线程
    virtual void shutdown();

    // 是否已经关闭
    bool isShutdown()
    {
        return m_bShutdown;
    }

    static MODULE_MAP &getModuleMap();

    // 获取模块栈
    static MODULE_QUEUE &getModuleQueue();

protected:
    // 等待所有模块完全加载后统一启动开启
    virtual void waitForStart();

    //消息接收任务；
    virtual void TaskMain(void *pBuff) = 0;

    // 注册模块消息
    virtual bool RegistMessage() = 0;

    // 起线程
    virtual pthread_t startupTask(PTHREAD_FUNC task, void *pThis);

    // 获取消息
    bool GetModuleMessage(MODULE_MESSAGE *&pMsg);

protected:
    static MODULE_MAP s_ObjList;
    static MODULE_QUEUE s_ModuleQueue;

    //开始标记
    UINT begin_flag;

    // 消息队列锁
    mutex m_msgMtx;

    // 消息信号量
    QSemaphore m_msgSem;

    //主任务开启信号量
    QSemaphore m_startSem;

    //线程句柄
    pthread_t m_nMainThreadID;

    // 是否关闭
    bool m_bShutdown;

    // 结束标记
    UINT end_flag;

    // 消息队列
    list<MODULE_MESSAGE *> m_MsgQue;
};

// 模块名称
#define MODULE_NAME(module_name)   \
    virtual string name() override \
    {                              \
        return #module_name;       \
    }

//模块初始化
extern bool LoadPlugin(string strModule);

//查询模块信息
extern IModule *QueryPlugin(string strModule);

//开启所有主任务
extern void startUpAll();

//关闭全部模块
extern void ShutdownAll();

// 设置任务运行优先级(实时/)
extern void setTaskRunType(int sched);

// 设置任务名称
extern void setCurrentTaskName(string thread_name);

// 毫秒延时
extern void mSleep(int ms);
