﻿#ifndef FACTORY_H
#define FACTORY_H
#include <functional>
#include <string>
#include <memory>
#include <map>
#include <iostream>
#include "base_module.h"

// 类工厂这里主要配合模块基类生成具体的模块实例
template <typename T>
class factory
{
public:
    /* N子类，这里会把对应子类添加到Map中 */
    template <typename N>
    struct register_t
    {
        template <typename... Args>
        register_t(const std::string& key, Args... args)
        {
            factory::instance()->_map.emplace(key, [=] { return new N(args...); });
        }
    };

    /*普通指针*/
    static auto produce(const std::string& key) -> T*
    {
        return factory::instance()->find(key);
    }

    /*unique_ptr指针*/
    static std::unique_ptr<T> unique_ptr(const std::string& key)
    {
        return std::unique_ptr<T>(produce(key));
    }

    /*shared_ptr指针*/
    static std::shared_ptr<T> shared_ptr(const std::string& key)
    {
        return std::shared_ptr<T>(produce(key));
    }

private:
    factory(){}
    factory(const factory&) = delete;
    factory(factory&&)      = delete;
    factory& operator=(const factory&) = delete;

    /* 单例模式 */
    static auto instance() -> factory<T>*
    {
        static factory<T> ins;
        return &ins;
    }

    auto find(const std::string& key) -> T*
    {
        if (_map.find(key) == _map.end())
        {
            //throw std::invalid_argument("key is not exist!");
            return nullptr;
        }

        return _map[key]();
    }
    std::map<std::string, std::function<T*()>> _map;
};

#define AF_OBJECT factory<IModule>
#define REGISTER_MODULE_VNAME(T) reg_msg_##T##_
#define REGISTER_MODULE(T, key, ...) static AF_OBJECT::register_t<T> REGISTER_MODULE_VNAME(T)(key, ##__VA_ARGS__);

#endif // FACTORY_H


