#include "CCameraManager.h"
#include "../socket/TcpServer.h"
#include "../base_module/factory.h"
#include "config/CConfigManager.h"
#include "timer/CTimerManager.h"
#include "device/CDeviceManager.h"
#include "socket/CWebSocketClient.h"
#include "file/CFileManager.h"
#include <QMutexLocker>
#include <assert.h>
#include "jt808/jt808_common.h"

#include "gps/CGPSManager.h"
#include "eSDKOBS.h"
#include "demo_common.h"
#include "securec.h"





////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////              CCameraManager              /////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

REGISTER_MODULE(CCameraManager, "CCameraManager")
LOG_DEFINITION(CCameraManager)

const QString CAMERA_CONFIG_FILE = "./config/cam_cfg.xml";
const QString VIDEO_DIR_NAME = "video/";
const QString IMAGE_DIR_NAME = "image/";

CCameraManager::CCameraManager()
    : QObject(),
      IModule(),
      m_pTcpModule(nullptr),
      m_pTimerMng(nullptr),
      m_pConfigMng(nullptr),
      m_pDevMng(nullptr),
      m_pWebSock(nullptr),
      m_pFileMng(nullptr),
      m_pCamDb(nullptr),
	  m_pGps(nullptr),
	  m_devideId("12345678")
{
    memset(&m_vctCamWID, 0, sizeof(m_vctCamWID));
	m_dis = 0.0;
    LOG_CREATE(CCameraManager, "camera");
}

CCameraManager::~CCameraManager()
{
    foreach (CCamConfig *pInfo, m_mapCamConfig)
    {
        SAFE_DELETE(pInfo);
    }
	obs_deinitialize();   
    std::cout << name() << std::endl;
}

//初始化操作
bool CCameraManager::Initialize()
{
    InitializeDB();
    m_pDevMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
    assert(m_pDevMng);
	m_pGps = dynamic_cast<CGPSManager *>(QueryPlugin("CGPSManager"));
    for (int i = 0; i < MAX_CAMERA_COUNT; i++)
    {
        m_vctCam[i].m_pCamMng = this;
    }

	if (m_pDevMng)
	{
		m_devideId = m_pDevMng->getDeviceID();
	}
	std::string de = m_devideId.toStdString();
	printf("%%%%%%%% initialize  devide  is %s\n", de.c_str());
	qDebug() << m_pDevMng->getDeviceID();
	
	kp = new CKafkaProducer;
	char topic[] = "bvai-detected-result";
    char brokers[] = "119.3.148.141:58007";
    int partition = 0;
	int ret = 0;
	ret = kp->init(topic, brokers, partition);
	
	if(ret != 0)
	{
		printf("Error: kp.init(): ret=%d;\n", ret);
	}
	
	obs_canned_acl canned_acl = OBS_CANNED_ACL_BUCKET_OWNER_FULL_CONTROL;
	/*------ obs init------*/
    set_obs_log_path("/var/log/OBS_SDK_C", false);                   //此行代码用于示例设置日志文件路径功能，在实际使用中请注释该行
    obs_initialize(OBS_INIT_ALL);
    set_online_request_max_count(10);
	
    //起主消息处理线程
    STARTUP_TASK(TaskMain)

    return true;
}

// 创建数据库
bool CCameraManager::InitializeDB()
{
    // 注册数据库
    m_pConfigMng = dynamic_cast<CConfigManager *>(QueryPlugin("CConfigManager"));
    if (m_pConfigMng)
    {
        m_pCamDb = m_pConfigMng->getDatabase(MODULE_CAMERA_MNG, CAMERA_DB_NAME);
        if (!m_pCamDb)
        {
            m_pCamDb = m_pConfigMng->regDatabase(MODULE_CAMERA_MNG, CAMERA_DB_NAME);
            m_pCamDb->regKey("video_save_interval", 600);
            m_pCamDb->save();
        }
    }

    return true;
}

// 保存数据库
bool CCameraManager::saveConfig()
{
    if (!m_pCamDb)
    {
        m_pCamDb->save();
        return true;
    }
    return false;
}

void CCameraManager::shutdown()
{
    m_webCamFinder.shutdown();
    IModule::shutdown();
}

// 恢复出厂配置回调函数
bool CCameraManager::restoreFactoryCallback(void *This, int type)
{
    CCameraManager *pThis = (CCameraManager *)This;
    return pThis->restoreFactory(type);
}

// 恢复出厂值
bool CCameraManager::restoreFactory(int type)
{
    for (int i = 0; i < MAX_CAMERA_COUNT; i++)
    {
        if (!m_vctCam[i].isCameraIdel())
        {
            m_vctCam[i].stopRun();
        }
        /* code */
    }
}

// 获取摄像头
CCameraDevice *CCameraManager::getCameraByIndex(int index)
{
    if (index >= 0 && index < MAX_CAMERA_COUNT)
    {
        return &(m_vctCam[index]);
    }
    return nullptr;
}

// 视频保存间隔
int CCameraManager::getVideoInterval()
{
    int interval = 600;
    if (m_pCamDb)
    {
        interval = m_pCamDb->getValue("video_save_interval").toInt();
    }
    return interval;
}

// 设置视频保存间隔
bool CCameraManager::setVideoInterval(int interval)
{
    if (interval < 60 || interval > 3600 * 4)
    {
        return false;
    }

    if (m_pCamDb)
    {
        m_pCamDb->setValue("video_save_interval", interval);
    }
}

// 设置相机配置
void CCameraManager::setCamConfig(CCamConfig &cam_cfg)
{
    m_mtxCamCfg.lock();
    if (m_mapCamConfig.find(cam_cfg.m_strCamDev) == m_mapCamConfig.end())
    {
        CCamConfig *pCam_cfg = new CCamConfig(cam_cfg);
        m_mapCamConfig[cam_cfg.m_strCamDev] = pCam_cfg;
    }
    else
    {
        *(m_mapCamConfig[cam_cfg.m_strCamDev]) = cam_cfg;
    }
    m_mtxCamCfg.unlock();
}

void CCameraManager::setVideoWId(int index, WId win_id)
{
    if (index >= 0 && index < MAX_CAMERA_COUNT)
    {
        m_vctCamWID[index] = win_id;
        m_vctCam[index].setWinIndex(index);
        m_vctCam[index].setWindowID(win_id);
    }
}



bool CCameraManager::RegistMessage()
{
    m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    if (m_pTcpModule)
    {
        m_pTcpModule->RegistMessageCallback(this, MODULE_CAMERA_MNG);
    }
    return true;
}

QString CCameraManager::getVideoPath()
{
    if (m_pFileMng)
    {
        return m_pFileMng->getDataPath() + "video/";
    }
    else
    {
        return "./video/";
    }
}

QString CCameraManager::getSnapshotPath()
{
    if (m_pFileMng)
    {
        return m_pFileMng->getDataPath() + "image/";
    }
    else
    {
        return "./image/";
    }
}
QString CCameraManager::getDevideId()
{
	return m_devideId;
}

//主消息任务，主要用于和其他模块之间的消息通讯
void CCameraManager::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskVideoMain");
    MODULE_MESSAGE *pMsg = nullptr;
    RegistMessage();
    waitForStart();

    //起主消息处理线程
    STARTUP_TASK(CameraTask);

    connect2Slots();

    while (!isShutdown())
    {
        if (GetModuleMessage(pMsg)) // get msg from message queue
        {
            switch (pMsg->iCmd)
            {
            case CMD_VIDEO_SET_CAM_CFG:
                procSetCamConfig(pMsg);
                break;
            case TIMER_FIND_CAM_DEVICE:
                STARTUP_TASK(tskCameraFind);
                break;
            case TIMER_SAVE_TO_NEW_VIDEO:
                procSave2NewFile(pMsg);
                break;
            case CMD_START_GB28181_VIDEO:
                procStartGB28181Video(pMsg);
                break;
            case CMD_STOP_GB28181_VIDEO:
                procStopGB28181Video(pMsg);
                break;
            case CMD_START_JT808_RT_VIDEO:
                procStartJt808RTVideo(pMsg);
                break;
            case CMD_JT808_RT_VIDEO_CONTROL:
                procJt808RTVideoControl(pMsg);
                break;
            case CMD_START_JT808_HIS_VIDEO:
                procStartJt808HisVideo(pMsg);
                break;
            case CMD_JT808_HIS_VIDEO_CONTROL:
                procJt808HisVideoControl(pMsg);
                break;
		    case CMD_TIMER_SAVE_TO_NEW_VIDEO_TEST:
				procSnapTest(pMsg);
				break;
			case CMD_TIMER_DEVICE_UPDATE:
				procDeviceId(pMsg);
				break;
			case CMD_TIMER_PHOTO_UPLOAD:
			//    procDistense(pMsg);
				//procSnapUpload(pMsg);
				break;
			
            default:
                break;
            }

            SAFE_DELETE(pMsg);
        }
    }
}

// 信号槽简易信号处理
void CCameraManager::connect2Slots()
{
    m_pWebSock = dynamic_cast<CWebSocketClient *>(QueryPlugin("CWebSocketClient"));
    m_pConfigMng = dynamic_cast<CConfigManager *>(QueryPlugin("CConfigManager"));
    m_pFileMng = dynamic_cast<CFileManager *>(QueryPlugin("CFileManager"));

    assert(m_pConfigMng);
    m_pConfigMng->regRestoreCallback(restoreFactoryCallback, (void *)this);

    m_pTimerMng = dynamic_cast<CTimerManager *>(::QueryPlugin("CTimerManager"));
    if (m_pTimerMng)
    {
        int cam_finder_interval = m_pCamDb->getValue("cam_finder_interval").toInt();
        int video_save_interval = m_pCamDb->getValue("video_save_interval").toInt();

        // 这里做个限制,最小查找时间60秒间隔,防止查找过于频繁
        if (cam_finder_interval < 60)
        {
            cam_finder_interval = 60;
        }

        m_pTimerMng->RegTimer(MODULE_CAMERA_MNG, TIMER_FIND_CAM_DEVICE, cam_finder_interval);
        m_pTimerMng->RegTimer(MODULE_CAMERA_MNG, TIMER_SAVE_TO_NEW_VIDEO, video_save_interval, TIMER_TYPE_INTEGRAL);
		m_pTimerMng->RegTimer(MODULE_CAMERA_MNG, CMD_TIMER_DEVICE_UPDATE, 60, TIMER_TYPE_INTEGRAL);
		//m_pTimerMng->RegTimer(MODULE_CAMERA_MNG, CMD_TIMER_PHOTO_UPLOAD, 1, TIMER_TYPE_INTEGRAL);
    }

    STARTUP_TASK(tskCameraFind);

    m_pDevMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
	m_pGps = dynamic_cast<CGPSManager *>(QueryPlugin("CGPSManager"));
	
    if (m_pFileMng)
    {
        makePath(m_pFileMng->getDataPath() + VIDEO_DIR_NAME);
        makePath(m_pFileMng->getDataPath() + IMAGE_DIR_NAME);
    }
}
std::string Int_to_String(guint64 n)

{

std::ostringstream stream;

stream<<n;  //n为int类型
std::string res = stream.str()+".jpg";
return res;

}

cJSON* CCameraManager::json_proc(jpgBuf *jb,int idx)
{
	if (jb == NULL)
		return NULL;
	cJSON*	cjson_callback=NULL;
	cjson_callback= cJSON_CreateObject();
	
	cJSON_AddStringToObject(cjson_callback, "ai_device_id", "test_0");
	cJSON_AddStringToObject(cjson_callback, "ai_version", "v1.1.1");
	
	QString did = m_pDevMng->getDeviceID();
	std::string strdid = did.toStdString();

	cJSON_AddStringToObject(cjson_callback, "device_id",strdid.c_str() );

	cJSON_AddNumberToObject(cjson_callback, "cam_id", idx);

	
	//cJSON_AddNumberToObject(cjson_callback,"frame_time", g_get_real_time());
	
	std::string rr = Int_to_String(g_get_real_time());
	cJSON_AddStringToObject(cjson_callback,"ai_model_type", "E99999");
	cJSON_AddStringToObject(cjson_callback,"frame_md5", rr.c_str());
	
	auto now = std::chrono::system_clock::now();
	time_t tt =std::chrono::system_clock::to_time_t(now);

	cJSON_AddNumberToObject(cjson_callback,"detected_time",(tt*1000) );
	cJSON_AddNumberToObject(cjson_callback,"frame_time", (tt*1000));
	cJSON*	cjson_frame_info = cJSON_CreateObject();
	cJSON_AddNumberToObject(cjson_frame_info,"width", jb->width);	
	cJSON_AddNumberToObject(cjson_frame_info,"height", jb->height);
	
	cJSON_AddItemToObject(cjson_callback, "frame_info", cjson_frame_info);
	
	cJSON*	cjson_frame_attach = cJSON_CreateObject();	
	cJSON_AddTrueToObject(cjson_frame_attach, "detected_attach");
	cJSON_AddItemToObject(cjson_callback, "detected_attach", cjson_frame_attach);
	cJSON*	cjson_attach = cJSON_CreateObject();
	cJSON_AddNumberToObject(cjson_attach, "frame_number", 1);
	cJSON_AddNumberToObject(cjson_attach, "num_object_meta", 1);
	cJSON_AddNumberToObject(cjson_attach, "batch_id", 1);
	
	cJSON*	cjson_obj = cJSON_CreateObject();	
	cJSON_AddNumberToObject(cjson_obj, "rubbish_id", 0);
	cJSON_AddStringToObject(cjson_obj, "class_name", "E99999");
	cJSON_AddNumberToObject(cjson_obj, "score", 0);			
	cJSON_AddNumberToObject(cjson_obj, "x_left", 0);
	cJSON_AddNumberToObject(cjson_obj, "y_left", 0);
	cJSON_AddNumberToObject(cjson_obj, "x_right", 0);
	cJSON_AddNumberToObject(cjson_obj, "y_right", 0);
	
	cJSON*	mi_arr = cJSON_CreateArray();
	cJSON_AddItemToArray(mi_arr,cjson_obj);
	cJSON_AddItemToObject(cjson_callback,"detected_info",mi_arr);
	
	
	cJSON*	gps_arr = cJSON_CreateArray();
	QString la = m_pGps->getLatitude();
	QString lo = m_pGps->getLongitude();
	
	
	std::string strla = la.toStdString();
	std::string strlo = lo.toStdString();
	
	
	cJSON_AddItemToArray(gps_arr, cJSON_CreateString(strla.c_str()));
	cJSON_AddItemToArray(gps_arr, cJSON_CreateString(strlo.c_str()));
	cJSON_AddItemToObject(cjson_attach,"gps",gps_arr);
	cJSON_AddNumberToObject(cjson_attach, "speed", m_pGps->getSpeedInfo());
	cJSON_AddNumberToObject(cjson_attach, "angle",m_pGps->getAngleInfo());
	
	cJSON_AddItemToObject(cjson_callback, "attach", cjson_attach);
	
	//char *infoj = cJSON_Print(cjson_callback);
	//LOG_INFO("json is %s\n", infoj);
	return cjson_callback;
}


void CCameraManager::procSnapTest(MODULE_MESSAGE *pMsg)
{

	jpgBuf *ret1 = m_vctCam[0].capSnapshotString();
	cJSON* rjson1 = json_proc(ret1, 0);
	jpgBuf *ret2 = m_vctCam[1].capSnapshotString();
	cJSON* rjson2 = json_proc(ret2, 1);
	jpgBuf *ret3 = m_vctCam[2].capSnapshotString();
	cJSON* rjson3 = json_proc(ret3, 2);
	jpgBuf *ret4 = m_vctCam[3].capSnapshotString();
	cJSON* rjson4 = json_proc(ret4, 3);
	int xx = 0;
	if (ret1 != NULL)
	{
	if (kp !=NULL)
		{
			xx = 1;
			char *json_result = cJSON_PrintUnformatted(rjson1);
			char *infoj = cJSON_Print(rjson1);
			printf("json is %s\n", infoj);
			kp->messagePushback(json_result,strlen(json_result),  ret1->jpg, ret1->size);
			//LOG_INFO("message push bak\n");
			delete ret1->jpg;
			delete ret1;
			cJSON_free(rjson1);
		}
	}	
	if (ret2 != NULL)
	{
	if (kp !=NULL)
		{
			xx = 1;
			char *json_result = cJSON_PrintUnformatted(rjson2);
			char *infoj = cJSON_Print(rjson2);
			printf("json is %s\n", infoj);
			kp->messagePushback(json_result,strlen(json_result),  ret2->jpg, ret2->size);
			//LOG_INFO("message push bak\n");
			delete ret2->jpg;
			delete ret2;
			cJSON_free(rjson2);
		}
	}

	if (ret3 != NULL)
	{
	if (kp !=NULL)
		{
			xx = 1;
			char *json_result = cJSON_PrintUnformatted(rjson3);
			char *infoj = cJSON_Print(rjson3);
			printf("json is %s\n", infoj);
			kp->messagePushback(json_result,strlen(json_result),  ret3->jpg, ret3->size);
			//LOG_INFO("message push bak\n");
			delete ret3->jpg;
			delete ret3;
			cJSON_free(rjson3);
		}
	}
	if (ret4 != NULL)
	{
	if (kp !=NULL)
		{
			xx = 1;
			char *json_result = cJSON_PrintUnformatted(rjson4);
			char *infoj = cJSON_Print(rjson4);
			printf("json is %s\n", infoj);
			kp->messagePushback(json_result,strlen(json_result),  ret4->jpg, ret4->size);
			//LOG_INFO("message push bak\n");
			delete ret4->jpg;
			delete ret4;
			cJSON_free(rjson4);
		}
	}
	

	emit notifyScreenshotTip(xx);

		//emit notifyScreenshotTip(0);
	
}
QString CCameraManager::genSnapshotPathName(int ind)
{
	QString devide_id = getDevideId();
	
	std::string de = devide_id.toStdString();
	printf("%%%%%%%% genSnapshotPathName  devide  is %s\n", de.c_str());
    QDateTime tm = QDateTime::currentDateTime();
    //QString str_name = QString("IMG_%1_Cam%2.jpg").arg(tm.toString("yyyy-MM-dd hh:mm:ss")).arg(m_winIdx);
	QString str_name = QString("%1/%2/%3/%4.jpg").arg(tm.toString("yyyy/M/d")).arg(m_devideId).arg(ind).arg(QString::number(tm.toTime_t()));
    return str_name;
}

static void progress_callback(uint64_t now, uint64_t total, void* callback_data)
{
    if (total)
    {
        printf("progress is %d%% \n", (now * 100) / total);
    }
}
int  showResponsePropertiesG = 1;
obs_status response_properties_callback(const obs_response_properties *properties, void *callback_data)
{

    if (properties == NULL)
    {
        printf("error! obs_response_properties is null!");
        if(callback_data != NULL)
        {
            obs_sever_callback_data *data = (obs_sever_callback_data *)callback_data;
            printf("server_callback buf is %s ,len is %d",
                data->buffer, data->buffer_len);
            return OBS_STATUS_OK;
        }else {
            printf("error! obs_sever_callback_data is null!");
            return OBS_STATUS_OK;
        }
    }

    if (!showResponsePropertiesG) {
        return OBS_STATUS_OK;
    }

#define print_nonnull(name, field)                                 \
    do {                                                           \
        if (properties-> field) {                                  \
            printf("%s: %s\n", name, properties-> field);          \
        }                                                          \
    } while (0)
    
    print_nonnull("ETag", etag);
    print_nonnull("expiration", expiration);
    print_nonnull("website_redirect_location", website_redirect_location);
    print_nonnull("version_id", version_id);
    print_nonnull("storage_class", storage_class);
    if (properties->last_modified > 0) {
        char timebuf[256] = {0};
        time_t t = (time_t) properties->last_modified;
        strftime(timebuf, sizeof(timebuf), "%Y-%m-%dT%H:%M:%SZ", gmtime(&t));
        printf("Last-Modified: %s\n", timebuf);
    }
    int i;
    for (i = 0; i < properties->meta_data_count; i++) {
        printf("x-amz-meta-%s: %s\n", properties->meta_data[i].name,
               properties->meta_data[i].value);
    }
    return OBS_STATUS_OK;
}


void put_buffer_complete_callback(obs_status status,
                                     const obs_error_details *error, 
                                     void *callback_data)
{
    put_buffer_object_callback_data *data = (put_buffer_object_callback_data *)callback_data;
    data->ret_status = status;
}


int put_buffer_data_callback(int buffer_size, char *buffer, void *callback_data)
{
    put_buffer_object_callback_data *data = 
        (put_buffer_object_callback_data *) callback_data;
    
    int toRead = 0;
    if (data->buffer_size) {
        toRead = ((data->buffer_size > (unsigned) buffer_size) ?
                    (unsigned) buffer_size : data->buffer_size);
        memcpy_s(buffer,buffer_size ,data->put_buffer + data->cur_offset, toRead);
    }
    
    uint64_t originalContentLength = data->buffer_size;
    data->buffer_size -= toRead;    
    data->cur_offset += toRead;
    if (data->buffer_size ) {
        printf("%llu bytes remaining ", (unsigned long long)data->buffer_size);
        printf("(%d%% complete) ...\n", 
            (int) (((originalContentLength - data->buffer_size) * 100) / originalContentLength));
    }

    return toRead;
}


int CCameraManager::Send_obs(jpgBuf *jb,QString pa)
{
	obs_options option;
    init_obs_options(&option);
    option.bucket_options.host_name = "obs.cn-east-2.myhuaweicloud.com";
    option.bucket_options.bucket_name = "ai5bucket";
    option.bucket_options.access_key = "GJZRCSRTJGSSFSVWPRKQ";
    option.bucket_options.secret_access_key = "zvdb0i8lIwPXCxSVOQCofMIBfec6w7W2Dm9hQ0wH";
	
	obs_put_properties put_properties;
    init_put_properties(&put_properties); 
    put_buffer_object_callback_data data;
    memset_s(&data, sizeof(data),0, sizeof(put_buffer_object_callback_data));
	
	data.put_buffer = jb->jpg;
	data.buffer_size = jb->size;
	obs_put_object_handler putobjectHandler =
    { 
        { &response_properties_callback, &put_buffer_complete_callback },
          &put_buffer_data_callback,
          &progress_callback
    };
	std::string ks = pa.toStdString();
	printf("key is %s\n", ks.c_str());
	
	put_object(&option, (char*)ks.c_str(), jb->size, &put_properties,0,&putobjectHandler,&data);

	if (OBS_STATUS_OK == data.ret_status) {
		return 1;
	}
	else
	{
		printf("put object %s from buffer failed(%s)%d.\n", ks.c_str(), obs_get_status_name(data.ret_status), data.ret_status);
		return 0;
	}
	
}


cJSON*  CCameraManager::proc_Obs(jpgBuf *jb,int idx)
{
	if (jb == NULL)
		return NULL;
	if (m_devideId == NULL || m_devideId == "12345678")
	{
		if (m_pDevMng)
		{
			m_devideId = m_pDevMng->getDeviceID();
		}
		printf("devided id is error\n");
		delete jb->jpg;
		delete jb;
		return NULL;
	}
	
	
	QString pa = genSnapshotPathName(idx);
	int ret = Send_obs(jb,pa);
	delete jb->jpg;
	delete jb;
	if (!ret)
		return NULL;
	
	QString pastr = "obs://ai5bucket/" + pa;
	return cJSON_CreateString(pastr.toStdString().c_str());

}


void CCameraManager::procSnapUpload(MODULE_MESSAGE *pMsg)
{
	jpgBuf *ret1 = m_vctCam[0].capSnapshotString();
	cJSON *json1 = proc_Obs(ret1,1);
	jpgBuf *ret2 = m_vctCam[1].capSnapshotString();
	cJSON *json2 = proc_Obs(ret2,2);
	jpgBuf *ret3 = m_vctCam[2].capSnapshotString();
	cJSON *json3 = proc_Obs(ret3,3);
	jpgBuf *ret4 = m_vctCam[3].capSnapshotString();
	cJSON *json4 = proc_Obs(ret4,4);
	
	if (json1!=NULL || json2!=NULL || json3!=NULL || json4!=NULL)
	{
		if (kp !=NULL)
		{
			
			cJSON*	cjson_obs=NULL;
			cjson_obs= cJSON_CreateObject();
			QString did = m_pDevMng->getDeviceID();
			std::string strdid = did.toStdString();

			cJSON_AddStringToObject(cjson_obs, "device_id",strdid.c_str() );
			cJSON*	gps_arr = cJSON_CreateArray();
			QString la = m_pGps->getLatitude();
			QString lo = m_pGps->getLongitude();
		
			std::string strla = la.toStdString();
			std::string strlo = lo.toStdString();
		
			cJSON_AddItemToArray(gps_arr, cJSON_CreateString(strla.c_str()));
			cJSON_AddItemToArray(gps_arr, cJSON_CreateString(strlo.c_str()));
			cJSON_AddItemToObject(cjson_obs,"gps",gps_arr);
			cJSON_AddNumberToObject(cjson_obs, "speed", m_pGps->getSpeedInfo());
			cJSON_AddNumberToObject(cjson_obs, "angle",m_pGps->getAngleInfo());
			
			 cJSON *array = cJSON_CreateArray(); 
			 if (json1 != NULL)
				 cJSON_AddItemToArray(array, json1);
			 if (json2 != NULL)
				 cJSON_AddItemToArray(array, json2);
			 if (json3 != NULL)
				 cJSON_AddItemToArray(array, json3);
			 if (json4 != NULL)
				 cJSON_AddItemToArray(array, json4);
			 
			cJSON_AddItemToObject(cjson_obs,"obs", array); 
			
			char *json_result = cJSON_PrintUnformatted(cjson_obs);
			char *infoj = cJSON_Print(cjson_obs);
			printf("obs no send json is %s \n", infoj);
			//kp->messagejsonPushback(json_result,strlen(json_result));
			
			cJSON_free(json_result);
					
		}
	}
	
	
	
	
	
}

void CCameraManager::procDistense(MODULE_MESSAGE *pMsg)
{
	if(m_pGps->isLegal())
	{
		float sp = m_pGps->getSpeedInfo();
		float dis = sp * 0.27*1;
		m_dis += dis;
		if (m_dis >= 10.0)
		{
			m_dis = 0.0;
			procSnapUpload(NULL);
		}
		
	}
	else{
		m_dis = 0.0;
		printf("m_pGps is not legal\n");
	}
}



void CCameraManager::procDeviceId(MODULE_MESSAGE *pMsg)
{
	m_devideId = m_pDevMng->getDeviceID();
}
// 开始gb28181推流
void CCameraManager::procStartGB28181Video(MODULE_MESSAGE *pMsg)
{
    assert(pMsg != nullptr);
    int idx = 0;
    int cam_idx = 0;
    int stream_code = 0;
    int rtp_port = 0;
    string rtp_ip = "";
    string ssrc = "";
    // 通信类型 int
    memcpy(&cam_idx, pMsg->szBuff + idx, sizeof(int));
    idx += sizeof(int);

    // 码流 int
    memcpy(&stream_code, pMsg->szBuff + idx, sizeof(int));
    idx += sizeof(int);

    // 端口 int
    memcpy(&rtp_port, pMsg->szBuff + idx, sizeof(int));
    idx += sizeof(int);

    // 对端IP 20
    rtp_ip = (char *)(pMsg->szBuff + idx);
    idx += 20;

    // SSRC
    ssrc = (char *)(pMsg->szBuff + idx);
    idx += 64;

    // 判断输入的INDEX 是否合法
    if (cam_idx < 0 || cam_idx >= MAX_CAMERA_COUNT)
    {
        cam_idx = 0;
    }

    m_vctCam[cam_idx].startGB28181Video(rtp_ip, rtp_port, ssrc, stream_code);
}

// 停止gb28181推流
void CCameraManager::procStopGB28181Video(MODULE_MESSAGE *pMsg)
{
    assert(pMsg != nullptr);
    int idx = 0;
    int channel = 0;

    // 通道号
    memcpy(&channel, pMsg->szBuff + idx, sizeof(int));
    idx += sizeof(int);

    if (channel >= 0 && channel < MAX_CAMERA_COUNT)
    {
        m_vctCam[channel].stopGB28181Video();
    }
}

// 开始JT808实时推流
void CCameraManager::procStartJt808RTVideo(MODULE_MESSAGE *pMsg)
{
    assert(pMsg != nullptr);
    string phone_num = "";

    int idx = 0;
    int channel = 0;
    int stream_code = 0;
    QString strIp;
    int tcp_port;
    int udp_port;
    int cam_idx = 0;

    if (m_pDevMng)
    {
        phone_num = m_pDevMng->getPhoneNumber().toString().toStdString();
    }

    // ip地址长度
    int ip_len = pMsg->szBuff[idx++];
    // ip地址
    strIp = QString((char *)&pMsg->szBuff[idx]).mid(0, ip_len);
    idx += ip_len;
    // tcp 端口
    tcp_port = two_char_to_short(&pMsg->szBuff[idx]);
    idx += 2;
    // udp 端口
    udp_port = two_char_to_short(&pMsg->szBuff[idx]);
    idx += 2;
    // 通道号
    channel = pMsg->szBuff[idx++];
    // 数据类型
    int date_type = pMsg->szBuff[idx++];
    // 码流类型
    stream_code = pMsg->szBuff[idx++];

    cam_idx = channel - 1;

    // 判断输入的INDEX 是否合法
    if (cam_idx < 0 || cam_idx >= MAX_CAMERA_COUNT)
    {
        cam_idx = 0;
    }

    m_vctCam[cam_idx].startJt808RTVideo(phone_num, strIp.toStdString(), tcp_port, udp_port, stream_code);
}

// jt808实时推流控制
void CCameraManager::procJt808RTVideoControl(MODULE_MESSAGE *pMsg)
{
    assert(pMsg != nullptr);
    int idx = 0;
    BYTE channel = pMsg->szBuff[idx++];
    BYTE ctrl_code = pMsg->szBuff[idx++];
    BYTE stop_code = pMsg->szBuff[idx++];
    BYTE stream_code = pMsg->szBuff[idx++];

    int cam_idx = channel - 1;

    // 判断输入的INDEX 是否合法
    if (cam_idx < 0 || cam_idx >= MAX_CAMERA_COUNT)
    {
        cam_idx = 0;
    }

    switch (ctrl_code)
    {
    case 0: //停止推流
    {
        // 0和2状态表示关闭视频
        if (stop_code == 0 || stop_code == 2)
        {
            m_vctCam[cam_idx].stopJt808RTVideo();
        }
        break;
    }
    case 1: // 换码流
    {
        m_vctCam[cam_idx].changeJt808RTStreamCode(stream_code);
        break;
    }
    case 2: // 暂停推流
    {
        m_vctCam[cam_idx].pauseJt808RTVideo();
        break;
    }
    case 3: // 继续推流
    {
        m_vctCam[cam_idx].continueJt808RTVideo();
        break;
    }
    case 4:
    default:
        break;
    }
}

// 开始JT808历史推流
void CCameraManager::procStartJt808HisVideo(MODULE_MESSAGE *pMsg)
{
    assert(pMsg != nullptr);
    string phone_num = "";
    // 目标IP
    // 目标tcp端口
    // 目标udp端口
    // 文件长度
    const int MAX_FILE_PATH_LEN = 260;
    int idx = 0;
    int channel = 0;
    int stream_code = 0;
    char szIP[20];
    char file_path[MAX_FILE_PATH_LEN];
    int tcp_port;
    int udp_port;
    int cam_idx = 0;
    int speed = 0;

    if (m_pDevMng)
    {
        phone_num = m_pDevMng->getPhoneNumber().toString().toStdString();
    }

    // ip地址这里取总长度20
    strncpy(szIP, (char *)&pMsg->szBuff[idx], 20);
    idx += 20;
    // tcp 端口
    tcp_port = two_char_to_short(&pMsg->szBuff[idx]);
    idx += 2;
    // udp 端口
    udp_port = two_char_to_short(&pMsg->szBuff[idx]);
    idx += 2;
    // 通道号
    channel = pMsg->szBuff[idx++];
    // 文件名,长度取
    strncpy(file_path, (char *)&pMsg->szBuff[idx], MAX_FILE_PATH_LEN);
    idx += MAX_FILE_PATH_LEN;
    // 播放倍速
    speed = pMsg->szBuff[idx++];

    //
    cam_idx = channel - 1;

    // 判断输入的INDEX 是否合法
    if (cam_idx < 0 || cam_idx >= MAX_CAMERA_COUNT)
    {
        cam_idx = 0;
    }

    // jt808 历史视频播放
    m_vctCam[cam_idx].startJt808FileVideo(phone_num, szIP, tcp_port, udp_port, channel, file_path);
}

// JT808历史推流控制
void CCameraManager::procJt808HisVideoControl(MODULE_MESSAGE *pMsg)
{
    assert(pMsg != nullptr);
    int idx = 0;
    BYTE channel = pMsg->szBuff[idx++];
    BYTE ctrl_code = pMsg->szBuff[idx++];
    BYTE stop_code = pMsg->szBuff[idx++];
    BYTE stream_code = pMsg->szBuff[idx++];

    int cam_idx = channel - 1;

    // 判断输入的INDEX 是否合法
    if (cam_idx < 0 || cam_idx >= MAX_CAMERA_COUNT)
    {
        cam_idx = 0;
    }

    switch (ctrl_code)
    {
    case 0: //停止推流
    {
        // 0和2状态表示关闭视频
        if (stop_code == 0 || stop_code == 2)
        {
            m_vctCam[cam_idx].stopJt808RTVideo();
        }
        break;
    }
    case 1: // 换码流
    {
        m_vctCam[cam_idx].changeJt808RTStreamCode(stream_code);
        break;
    }
    case 2: // 暂停推流
    {
        m_vctCam[cam_idx].pauseJt808RTVideo();
        break;
    }
    case 3: // 继续推流
    {
        m_vctCam[cam_idx].continueJt808RTVideo();
        break;
    }
    case 4:
    default:
        break;
    }
}

// 处理摄像头配置变更
void CCameraManager::procSetCamConfig(MODULE_MESSAGE *pMsg)
{
    int index = 0;
    CCamConfig cam_cfg;
    int offset = 0;
    memcpy(&index, pMsg->szBuff + offset, sizeof(int));
    offset += sizeof(int);

    memcpy(&cam_cfg, pMsg->szBuff + offset, sizeof(cam_cfg));
    offset += sizeof(cam_cfg);

    if (index >= 0 && index < MAX_CAMERA_COUNT)
    {
        m_vctCam[index].setCamConfig(cam_cfg);
    }
}

void CCameraManager::procSave2NewFile(MODULE_MESSAGE *pMsg)
{
    QDateTime tmBegin;
    QDateTime tmEnd;
    int interval = m_pCamDb->getValue("video_save_interval").toInt();
    memcpy(&tmBegin, pMsg->szBuff, sizeof(tmBegin));
    tmEnd = tmBegin.addSecs(interval);

    for (int i = 0; i < MAX_CAMERA_COUNT; i++)
    {
        if (!m_vctCam[i].isCameraIdel())
        {
            m_vctCam[i].save2NewFile(tmBegin, tmEnd);
        }
    }
}

//实时摄像头任务，实时摄像头任务切换相关
void CCameraManager::CameraTask(void *pBuff)
{
    int i = 0;
    for (int i = 0; i < MAX_CAMERA_COUNT; i++)
    {
        m_vctCam[i].startSnapshotTask();
    }
}

// 视频保存模式：按时间长度保存（1）; 按文件大小保存(2)
void CCameraManager::onSaveModeChanged(int mode)
{
    qDebug() << "onSaveModeChanged:" << mode;
}

// 数据位置
void CCameraManager::onDataPathChanged(QString path)
{
    qDebug() << "onDataPathChanged:" << path;
}

// 命名规则
void CCameraManager::onNameRuleChanged(QString rule)
{
    qDebug() << "onSaveInternalChanged:" << rule;
}

// 按时间长度保存的时间间隔(分钟)
void CCameraManager::onSaveInternalChanged(int val)
{
    qDebug() << "onSaveInternalChanged:" << val;
}

// 按文件大小保存的文件大小(MB)
void CCameraManager::onSaveSizeChanged(int val)
{
    qDebug() << "onSaveSizeChanged:" << val;
}

// 获取指定索引的设备地址
bool CCameraManager::onCamCfgChanged(int idx, CCamConfig &cam_cfg)
{
    qDebug() << "onCamCfgChanged:" << idx;
}

// 拆帧间隔
void CCameraManager::onFrameSplittingIntervalChanged(int frameInverval)
{
    qDebug() << "onFrameSplittingIntervalChanged:" << frameInverval;
}

// 查找局域网ip
void CCameraManager::tskCameraFind(void *pBuff)
{
    assert(m_pConfigMng);
    static bool bFinding = false;
    if (!isShutdown() && bFinding == false)
    {
        m_mtxCamFind.lock();
        bFinding = true;
        m_webCamFinder.StopFind();
        m_webCamFinder.SetDiscoveryInfoCB(CCameraManager::updateCameraCallback, this);
        m_webCamFinder.StartFind();
        m_mtxCamFind.unlock();
    }
    bFinding = false;
}

// 获取空闲摄像头窗口
CCameraDevice *CCameraManager::getIdelCamera(int &index)
{
    for (int i = 0; i < MAX_CAMERA_COUNT; i++)
    {
        if (m_vctCam[i].isCameraIdel())
        {
            index = i;
            return &m_vctCam[i];
        }
    }
    return nullptr;
}

// 获取指定源的摄像头窗口
CCameraDevice *CCameraManager::getCameraBySrc(QString strSrc)
{

    for (int i = 0; i < MAX_CAMERA_COUNT; i++)
    {
        if (m_vctCam[i].getCamDev() == strSrc)
        {
            return &m_vctCam[i];
        }
    }
    return nullptr;
}

// 根据IP地址获取推荐的显示位
int CCameraManager::getPrefIdxByIP(QString strDev)
{
    int PrefIdx = 0;
    QStringList list = strDev.split('.');
    if (list.size() == 4)
    {
        PrefIdx = max(0, list[2].toInt() - 1);
    }
    return PrefIdx;
}
void CCameraManager::updateCamerapDevice(QString strDev)
{
    // 先判断指定IP是否在使用的设备列表中
    CCameraDevice *pCamera = getCameraBySrc(strDev);
    // 如果在，直接返回
    if (pCamera)
    {
        return;
    }
    // 如果不在,将摄像头放入一个指定的表
    else
    {
        QMutexLocker lock(&m_mtxCamera);
        CAM_CONFIG_MAP::Iterator it = m_mapCamConfig.find(strDev);
        CCamConfig cam_cfg(strDev);
        // 获取对应IP的配置信息
        if (it != m_mapCamConfig.end())
        {
            cam_cfg = *(it.value());
        }
        // 没有配置则初始化配置
        else
        {
            //
            cam_cfg.m_PrefIdx = getPrefIdxByIP(strDev);
            cam_cfg.setCamDev(strDev);
            setCamConfig(cam_cfg);
        }

        CCameraDevice *pDevice = nullptr;
        // 确认偏好索引的摄像头位置是否空闲
        int pref_idx = cam_cfg.m_PrefIdx;
        // 如果空闲，直接占用这个展示位
        if (pref_idx < MAX_CAMERA_COUNT && pref_idx >= 0 && m_vctCam[pref_idx].isCameraIdel())
        {
            pDevice = &m_vctCam[pref_idx];
            pDevice->m_PrefIdx = pref_idx;
        }
        // 如果不空闲，则获取一个空闲的摄像头位置
        else
        {
            pDevice = this->getIdelCamera(pref_idx);
            pDevice->m_PrefIdx = pref_idx;
        }

        // 开始运行
        if (pDevice)
        {
            pDevice->setCamConfig(cam_cfg);
            pDevice->setSource(cam_cfg.getVideoMain());
            pDevice->startRun();

            // TODO 广播摄像头上线事件
            MODULE_MESSAGE msg;
            int idx = 0;
            bool online = 1;
            msg.iCmd = CMD_CAM_ONLINE_OFFLINE;
            memcpy(msg.szBuff, &online, sizeof(bool));
            idx += sizeof(bool);
            memcpy(msg.szBuff + idx, &pDevice->m_PrefIdx, sizeof(int));
            idx += sizeof(int);

            m_pTcpModule->boardcastMessage(msg);
        }
    }
}

// 刷新设备列表
void CCameraManager::updateCameraCallback(DEVICE_IPC_INFO *pInfo, void *lpContext)
{
    CCameraManager *pThis = (CCameraManager *)lpContext;
    QString strDev = pInfo->IP;
    if (pThis != nullptr)
    {
        pThis->updateCamerapDevice(strDev);
    }
}

// 实时视频rtmp推送
bool CCameraManager::realtimeVideo(QVariantMap &prop_arr)
{
    bool bSuccess = false;
    // pushUrl	String	推流地址
    QString pushUrl = prop_arr["pushUrl"].toString();

    int cam_idx = max(0, prop_arr["camIdx"].toInt() - 1);

    if (cam_idx >= 0 && cam_idx < MAX_CAMERA_COUNT)
    {
        bSuccess = m_rtmpResponser.startRtmpRequire(m_vctCam[cam_idx].getVideoMain(), pushUrl);
    }

    onRealtimeVideo(pushUrl, bSuccess);

    // 开始推流处理
    return true;
}

// 实时视频推流的结果反馈
bool CCameraManager::onRealtimeVideo(QString pushUrl, bool bSuccess)
{
    assert(m_pWebSock);
    QVariantMap prop_arr;

    prop_arr.insert("msgName", "onRealtimeVideo");
    // pushUrl	String	推流地址
    prop_arr.insert("pushUrl", pushUrl);

    int retCode = bSuccess ? 1 : 0;
    // retCode	number	返回码，0表示成功
    prop_arr.insert("retCode", retCode);

    // message	String	返回消息
    prop_arr.insert("message", "");

    if (!m_pWebSock->sendMessage(prop_arr))
    {
        return false;
    }
    return true;
}

// 停止实时视频推流
bool CCameraManager::stopRealtimeVideo(QVariantMap &prop_arr)
{
    // pushUrl	String	推流地址
    QString pushUrl = prop_arr["pushUrl"].toString();

    m_rtmpResponser.stopRtmpRequire();
    // 停止推流处理
    return true;
}

// 抓拍
bool CCameraManager::snapShot(QVariantMap &prop_arr)
{
    // interval	number	多张抓拍之间的时间间距
    m_iSnapInterval = (int)prop_arr["interval"].toFloat();
    // number	number	抓拍的张数
    m_iSnapCount = (int)prop_arr["number"].toFloat();

    m_strSnapList.clear();

    if (m_iSnapCount > 0)
    {
        for (int i = 0; i < MAX_CAMERA_COUNT; i++)
        {
            if (!m_vctCam[i].isCameraIdel())
            {
                m_bSnapFinished[i] = false;
                m_vctCam[i].snapshot(m_iSnapInterval, m_iSnapCount);
            }
        }
    }
    else
    {
        onSnapShot();
    }

    return true;
}

bool CCameraManager::snapShotFinilished(int cam_idx, QString strFileList)
{
    bool bAllFinished = true;
    m_strSnapList += strFileList;

    if (cam_idx >= 0 && cam_idx < MAX_CAMERA_COUNT)
    {
        m_bSnapFinished[cam_idx] = true;
    }

    for (int i = 0; i < MAX_CAMERA_COUNT; i++)
    {
        if (!m_vctCam[i].isCameraIdel() && m_bSnapFinished[i] == false)
        {
            bAllFinished = false;
            break;
        }
    }

    if (bAllFinished)
    {
        onSnapShot();
    }
}
// 抓拍的结果反馈
bool CCameraManager::onSnapShot()
{
    assert(m_pWebSock);
    QVariantMap prop_arr;

    prop_arr.insert("msgName", "onSnapShot");
    // interval	number	多张抓拍之间的时间间距
    prop_arr.insert("interval", QString("%1").arg(m_iSnapInterval));

    // number	number	抓拍的张数
    prop_arr.insert("number", QString("%1").arg(m_iSnapCount));

    // fileList	Array	文件列表；可以直接下载的url的列表
    prop_arr.insert("fileList", m_strSnapList);

    if (!m_pWebSock->sendMessage(prop_arr))
    {
        return false;
    }
    return true;
}
