#ifndef CFILEMANAGER_H
#define CFILEMANAGER_H
#include "base_module.h"
#include <QObject>
#include <QMultiMap>
#include <QDateTime>
#include "alarm/CAlarmManager.h"
#include <QVector>
#include <QStorageInfo>
#include <QMutex>
#include "config/CConfigManager.h"

class ITcpModule;
class CWebSocketClient;
class CConfigManager;

enum FILE_TYPE
{
    FILE_TYPE_VIDEO = 0,
    FILE_TYPE_IMAGE,
    FILE_TYPE_UNKNOWN,
};

// 文件节点
struct CCamFileNode
{
    CCamFileNode();
    CCamFileNode(QString file_name);
    // 文件节点=>字符存
    QString toString();
    // 从字符串转出
    bool fromString(QString info);
    // 是否有效
    bool isValid();

    // 文件名
    QString fileName;
    // 开时时间
    QDateTime tmBegin;
    // 结束时间
    QDateTime tmEnd;
    // 锁定状态
    bool bFileLocked;
    // 告警信息
    uint64_t alarms;
    // 通道号
    BYTE channel;
    // 媒体类型
    BYTE media_type;
    // 码流类型
    BYTE stream_code;
    // 存储类型
    BYTE storage_type;
    // 文件大小
    DWORD file_size;
};

struct FILE_QUERY_INFO
{
    // 通道号
    BYTE channel;
    // 开时时间
    uint64_t tmBegin;
    // 结束时间
    uint64_t tmEnd;
    // 告警状态
    uint64_t alarm_status;
    // 媒体类型
    BYTE media_type;
    // 码流
    BYTE stream_code;
    // 存储介质
    BYTE storage_type;

    FILE_QUERY_INFO() {}
};

typedef QMap<QString, CCamFileNode *> FILE_LIST_BY_NAME;
typedef QMultiMap<uint, CCamFileNode *> FILE_LIST_BY_TIME;

typedef QVector<CCamFileNode> VCT_FILE_LIST;

// 文件节点管理（分视频和图像）
class CFileNodeMangaer
{
public:
    CFileNodeMangaer(QString strList, QString m_strFliter);
    // 获取视频列表
    QString getFileList(QDateTime &tmBegin, QDateTime &tmEnd);

    bool queryFileList(FILE_QUERY_INFO query, VCT_FILE_LIST &file_lst);

    // 添加文件节点
    void addFileNode(CCamFileNode &file_node);

    // 删除文件节点
    bool delFileNode(QString path_name);

    // 保存文件节点
    bool saveFileList();

    // 设置根目录
    void setDatePath(QString data_path) { m_strRootPath = data_path; }

    // 获取第一个有效的删除节点
    CCamFileNode *getFirstNodeForDel();

    // 导入文件节点
    bool loadFileList();

    // 恢复出厂设置
    void restoreFactory();

protected:
    // 检查列表文件
    bool checkFileList();

    // 获取根目录
    QString getDataPath() { return m_strRootPath; }

    // 重建文件列表
    bool rebuildFileList();

    // 查询文件大小
    int getFileSize(CCamFileNode &node);

    // 文件map表
    FILE_LIST_BY_TIME m_mapFlieListByTime;
    FILE_LIST_BY_NAME m_mapFlieListByName;
    // 保护信号量
    QMutex m_mtxFileList;
    // 根目录
    QString m_strRootPath;
    // 列表名
    QString m_strListName;
    // 后缀过滤
    QString m_strFliter;
    // 恢复锁
    bool m_bRestorFactoryLocked;
};

// 文件管理模块
class CFileManager : public QObject, public IModule
{
    Q_OBJECT
public:
    MODULE_NAME(CFileManager)
    explicit CFileManager(QObject *parent = nullptr);
    ~CFileManager();

    //初始化操作
    virtual bool Initialize();
    //模块注册接口
    bool RegistMessage();

    // 服务器要求获取录像文件的文件列表
    bool getVideoRecordList(QVariantMap &prop_arr);

    // 删除文件
    bool fileDelete(QVariantMap &prop_arr);

    // 获取硬盘类型 1表示SD卡，2表示硬盘，3表示SSD卡 先打桩
    int getDisktype();

    // 获取磁盘空间状态。剩余空间，单位MB 先打桩
    float getFreeSpaceMB();

    // 获取磁盘空间。
    float getDiskSpaceMB();

    // 获取磁盘空间状态。剩余空间，单位% 先打桩
    float getFreeSpacePrecent();

    // 获取数据路径
    QString getDataPath();

    // 恢复出厂配置回调函数
    static bool restoreFactoryCallback(void *This, int type);

    // 恢复出厂值
    bool restoreFactory(int type);

    // 查询文件列表
    bool queryFileList(FILE_QUERY_INFO query_info, VCT_FILE_LIST &file_list);

    QString getPathName(QString file_name) { return getDataPath()+file_name;}
Q_SIGNALS:
    //

public Q_SLOTS:

protected:
    //主消息任务，主要用于和其他模块之间的消息通讯
    THREAD_DEFINITION(CFileManager, TaskMain)
    //列表重建任务，系统启动过程中运行;
    THREAD_DEFINITION(CFileManager, TaskRebuildFileList)

    //主消息任务，主要用于和其他模块之间的消息通讯
    virtual void TaskMain(void *pBuff);

    //列表重建任务，系统启动过程中运行;
    virtual void TaskRebuildFileList(void *pBuff);

    // 重置数据
    bool RestoreDataset();

    // 删除文件消息处理
    void ProcDelFiles(MODULE_MESSAGE *pMsg);

    // 添加文件到列表
    bool addFile2List(MODULE_MESSAGE *pMsg);

    //get default disk
    QString getDefDisk();

    // 获取存储信息
    bool getStorageInfo(QString device, QStorageInfo &st);

    // 检查路径
    bool checkPath(QString strPath);

    // 生成数据路径
    void makePaths();

    // 返回删除文件结果
    bool onFileDelete(QVariantMap &prop_arr, int result);

    // 删除文件节点
    bool delFileNodeByName(QString path_name);

    // 管理磁盘空间
    void managerDiskSpace();

    // 创建数据库
    bool createDatebase();

    // 恢复数据库
    bool restoreFromDB();

    // 获取文件类型
    FILE_TYPE getFileTYPE(QString strname);

protected:
    // 依赖模块
    ITcpModule *m_pTcpModule;
    CWebSocketClient *m_pWebSock;
    CConfigManager *m_pConfigMng;

    // 磁盘路径
    QString m_strDatapath;

    // 磁盘信息
    QStorageInfo m_dataStorage;

    // 剩余空间门限
    float m_fMinFreeSpaceRate;

    // 视频文件管理
    CFileNodeMangaer m_vidFileMng;

    // 图像文件管理
    CFileNodeMangaer m_imgFileMng;

    // 数据库
    CDatebase *m_pDbFile;

    // 恢复出厂值锁定
    bool m_bRestoreFactoryLocked;
};

#endif // CFILEMANAGER_H
