#include "CGB28181Client.h"
#include <netinet/in.h>
#include <sstream>
#include "eXosip2/eXosip.h"
#include <QFile>

namespace GB28181
{
    CGB28181Client::CGB28181Client()
        : m_excontext(nullptr),
          m_eventId(1000000000),
          m_bRegok(false),
          m_incNumber(0)
    {
    }

    CGB28181Client::~CGB28181Client()
    {
        eXosip_quit(m_excontext);
    }

    bool CGB28181Client::Initialize()
    {
        if (!loadConfig())
        {
            return false;
        }

        m_excontext = eXosip_malloc();

        int ret = eXosip_init(m_excontext);
        if (ret != 0)
        {
            //LOG_INFO << "eXosip_init failed, ret=" << ret;
            return false;
        }
        if (!startListen("HeNeng AI"))
        {
            return false;
        }

        ostringstream from_uri;
        ostringstream proxy_uri;

        // local ip & port
        char ip_addr[64] = {0};
        eXosip_guess_localip(m_excontext, AF_INET, ip_addr, sizeof(ip_addr));
        //spdlog::info("local ip is {}", m_localIP);
        m_localIP = ip_addr;

        from_uri << "sip:" << m_DeviceID.toStdString() << "@" << m_localIP.toStdString() << ":" << m_localPort.toInt();
        proxy_uri << "sip:" << m_ServerID.toStdString() << "@" << m_serverIP.toStdString() << ":" << m_serverPort.toInt();

        m_localSip = from_uri.str();
        m_remoteSip = proxy_uri.str();

        //LOG_INFO << "eXosip_init successfully!";
        if (0 != RegisterOnline())
        {
            return false;
        }

        return true;
    }

    /*******************************************
    函数功能:开始SIP侦听
    入口参数: rid 注册ID
    出口参数: 0 成功
    备    注: 需要等待初始化完成
    *******************************************/
    bool CGB28181Client::startListen(const std::string &user_agent)
    {
        int ret = eXosip_listen_addr(m_excontext, IPPROTO_UDP,
                                     NULL, m_localPort.toInt(), AF_INET, 0);
        if (ret != OSIP_SUCCESS)
        {
            eXosip_quit(m_excontext);
            //LOG_INFO << "eXosip_listen_addr failed, ret: " << ret;
            return false;
        }

        if (user_agent.length() > 0)
        {
            eXosip_set_user_agent(m_excontext, user_agent.c_str());
        }

        return true;
    }

    int CGB28181Client::RegisterOnline()
    {
        osip_message_t *register_message = nullptr;
        exosip_locker locker(m_excontext);
        int rid = eXosip_register_build_initial_register(m_excontext,
                                                         m_localSip.c_str(),
                                                         m_remoteSip.c_str(),
                                                         m_localSip.c_str(), 3600, &register_message);
        if (nullptr == register_message)
        {
            //spdlog::error("eXosip_register_build_initial_register failed");
            return -1;
        }

        eXosip_add_authentication_info(m_excontext, m_ServerID.toStdString().c_str(), m_ServerID.toStdString().c_str(),
                                       m_ServerPwd.toStdString().c_str(), "MD5", nullptr);

        int r = eXosip_register_send_register(m_excontext, rid, register_message);
        if (r != 0)
        {
            //LOG_ERROR << "send register failed, ret: " << r;
            return -1;
        }
        return 0;
    }
    /*******************************************
    函数功能:接收sip消息事件函数
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    int CGB28181Client::recvEvents()
    {
        eXosip_event_t *exosip_event = nullptr;
        SIP_ENENT_PTR sipg_event = nullptr;
        try
        {

            exosip_event = eXosip_event_wait(m_excontext, 0, 4);
            if (exosip_event == nullptr)
            {
                return 0;
            }

            sipg_event = createEvent(m_excontext, exosip_event);
            if (nullptr == sipg_event)
            {
                return 0;
            }
            
            m_mtxQue.lock();
            m_eventQueue.enqueue(sipg_event);
            m_mtxQue.unlock();
        }
        catch (const std::exception &e)
        {
            return -1;
        }

        //LOG_DEBUG << "Push event: " << sipg_event->name << ", id=" << sipg_event->id << " to queue successfully";
        return 0;
    }

    /*******************************************
    函数功能:处理sip事件
    入口参数:
    出口参数: 成功返回0
    备    注:和接收函数处在两个不同的任务防止接收任务被堵塞
    *******************************************/
    int CGB28181Client::processEvents()
    {
        const char *event_name;
        uint64_t event_id;

        if (m_eventQueue.size() == 0)
        {
            return 0;
        }

        SIP_ENENT_PTR sipg_event;
        m_mtxQue.lock();
        sipg_event = m_eventQueue.dequeue();
        m_mtxQue.unlock();
        event_name = sipg_event->name;
        event_id = sipg_event->id;

        sipg_event->proc(sipg_event);
        eXosip_event_free(sipg_event->exevent);
        return 0;
    }

    /*******************************************
    函数功能: 创建事件节点
    入口参数:eXosip_t *exosip_context,  会话实体指针
            eXosip_event_t *exosip_event 事件指针
    出口参数:SIP_ENENT_PTR 事件指针
    备    注: 在接收函数中封装的事件节点信息，方便和处理函数通信
    *******************************************/
    SIP_ENENT_PTR CGB28181Client::createEvent(eXosip_t *exosip_context, eXosip_event_t *exosip_event)
    {
        if (exosip_event == nullptr)
        {
            return nullptr;
        }

        if (exosip_event->type < EXOSIP_REGISTRATION_SUCCESS || exosip_event->type > EXOSIP_NOTIFICATION_GLOBALFAILURE)
        {
            return nullptr;
        }

        SIP_ENENT_PTR event(new SIP_ENENT); // = std::make_shared(SipEvent)();
        EventNameProcPair pair = getEventProc(exosip_event->type);
        if (pair.name == nullptr)
        {
            return nullptr;
        }

        event->name = pair.name;
        event->proc = pair.proc;
        event->excontext = exosip_context;
        event->exevent = exosip_event;
        event->id = m_eventId++;

        return event;
    }

    /*******************************************
    函数功能:注册成功处理
    入口参数: rid 注册ID
    出口参数: 0 成功
    备    注:
    *******************************************/
    int CGB28181Client::OnRegisterSuccess(int rid)
    {
        return 0;
    }

    /*******************************************
    函数功能:PTZ控制,
    入口参数: control_cmd_t &ctrlcmd,    控制字
             string &devid              设备ID
    出口参数: 0 成功
    备    注:涉及到硬件控制,需要在派生类中实现通信操作用于控制云台
    *******************************************/
    int CGB28181Client::HandlePTZControl(DEVICE_CONTROL_CMD &ctrlcmd, string &devid)
    {
        return 0;
    }

    /*******************************************
    函数功能:设备控制
    入口参数: rid 注册ID
    出口参数: 0 成功
    备    注:涉及到硬件控制,需要在派生类中实现通信操用于控制设备
    *******************************************/
    int CGB28181Client::HandleDeviceControl(MANSCDP_DEV_CTRL_CMD cmd, string &devid, MANSCDP_SWITCH_STATUS onoff)
    {
        return 0;
    }

    /*******************************************
    函数功能:处理服务端消息响应
    入口参数: string &reqid, 流水号
            int status_code 状态码
    出口参数: 0 成功
    备    注:
    *******************************************/
    int CGB28181Client::onMsgResponse(string &reqid, int status_code)
    {
        return 0;
    }

    /*******************************************
    函数功能:发送请求
    入口参数: string context_type,  类型
             string msg            消息内容
    出口参数: 0 成功
    备    注:
    *******************************************/
    int CGB28181Client::sendRequest(string context_type, string msg)
    {
        int ret = 0;
        osip_message_t *request = createMsg();
        if (request == nullptr)
        {
            return -1;
        }

        ret = osip_message_set_content_type(request, context_type.c_str());
        if (0 != ret)
        {
            return ret;
        }

        ret = osip_message_set_body(request, msg.c_str(), strlen(msg.c_str()));
        if (0 != ret)
        {
            return ret;
        }

        exosip_locker locker(m_excontext);
        return eXosip_message_send_request(m_excontext, request);
    }

    /*******************************************
    函数功能:向sip终端发送消息/响应
    入口参数: rid 注册ID
    出口参数: 0 成功
    备    注:
    *******************************************/
    int CGB28181Client::sendResponse(eXosip_event_t *exevent, string context_type, string msg)
    {
        int ret = 0;
        osip_message_t *answer = createMsg();

        if (answer == nullptr)
        {
            return -1;
        }

        ret = osip_message_set_content_type(answer, context_type.c_str());
        if (0 != ret)
        {
            return ret;
        }

        ret = osip_message_set_body(answer, msg.c_str(), strlen(msg.c_str()));
        if (0 != ret)
        {
            return ret;
        }

        exosip_locker locker(m_excontext);
        eXosip_message_send_answer(m_excontext, exevent->tid, 200, answer);
        return 0;
    }

    /*******************************************
    函数功能:向sip终端发送简单响应
    入口参数: rid 注册ID
    出口参数: 0 成功
    备    注:
    *******************************************/
    int CGB28181Client::sendResponseOk(eXosip_event_t *evt)
    {
        int ret = 0;
        auto answer = evt->request;
        exosip_locker locker(m_excontext);

        ret = eXosip_message_build_answer(m_excontext, evt->tid, 200, &answer);
        if (0 != ret)
        {
            return ret;
        }

        return eXosip_message_send_answer(m_excontext, evt->tid, 200, answer);
    }

    /*******************************************
    函数功能:创建sip消息实例
    入口参数: 
    出口参数: osip_message_t * 消息指针
    备    注:
    *******************************************/
    osip_message_t *CGB28181Client::createMsg()
    {

        osip_message_t *request = nullptr;
        auto status = eXosip_message_build_request(m_excontext, &request, "MESSAGE", m_remoteSip.c_str(), m_localSip.c_str(), nullptr);
        if (OSIP_SUCCESS != status)
        {
            //spdlog::error("build request failed: {}", status);
        }

        return request;
    }
    /*******************************************
    函数功能:发送保活消息
    入口参数:
    出口参数: 0 成功
    备    注:
    *******************************************/
    int CGB28181Client::keepAlive()
    {
        stringstream ss;
        ss << "<?xml version=\"1.0\"?>\r\n";
        ss << "<Notify>\r\n";
        ss << "<CmdType>Keepalive</CmdType>\r\n";
        ss << "<SN>" << getIncNumber() << "</SN>\r\n";
        ss << "<DeviceID>" << m_DeviceID.toStdString() << "</DeviceID>\r\n";
        ss << "<Status>OK</Status>\r\n";
        ss << "</Notify>\r\n";

        return sendRequest("Application/MANSCDP+xml", ss.str());
    }

    /*******************************************
    函数功能:超时检查,主要是心跳超时检查
    入口参数:
    出口参数: 0 成功
    备    注:
    *******************************************/
    int CGB28181Client::checkTimeOut()
    {
        return 0;
    }


    // TODO 结束视频通话
    int CGB28181Client::onExosipCall_closed(const SIP_ENENT_PTR &evt)
    {
        int cam_idx = 0;

        const char *dest_id = evt->exevent->request->to->url->username;
        cam_idx = getCamIndexByID(dest_id);

        notifyStopVideo(cam_idx);
    }

    /* INVITE related events within calls */
    int CGB28181Client::onExosipCall_invite(const SIP_ENENT_PTR &evt)
    {
        int cam_idx = 0;
        //spdlog::info("got CALL_INVITE");
        auto sdp_msg = eXosip_get_remote_sdp(m_excontext, evt->exevent->did);
        if (!sdp_msg)
        {
            //spdlog::error("eXosip_get_remote_sdp failed");
            return -1;
        }

        auto connection = eXosip_get_video_connection(sdp_msg);
        if (!connection)
        {
            //spdlog::error("eXosip_get_video_connection failed");
            return -1;
        }

        string rtp_ip = connection->c_addr;

        auto video_sdp = eXosip_get_video_media(sdp_msg);
        if (!video_sdp)
        {
            //spdlog::error("eXosip_get_video_media failed");
            return -1;
        }

        int rtp_port = atoi(video_sdp->m_port);

        //spdlog::info("rtp server: {}:{}", rtp_ip, rtp_port);

        string rtp_protocol = video_sdp->m_proto;

        //spdlog::info("rtp protocol: {}", rtp_protocol);

        osip_body_t *sdp_body = NULL;
        osip_message_get_body(evt->exevent->request, 0, &sdp_body);
        if (nullptr == sdp_body)
        {
            //spdlog::error("osip_message_get_body failed");
            return -1;
        }

        string body = sdp_body->body;
        auto y_sdp_first_index = body.find("y=");
        auto y_sdp = body.substr(y_sdp_first_index);
        auto y_sdp_last_index = y_sdp.find("\r\n");
        string ssrc = y_sdp.substr(2, y_sdp_last_index - 1);
        //spdlog::info("ssrc: {}", ssrc);

        if(evt->exevent->response)
        {
            char dest_id[SIP_ID_LEN+1] = {0};
            QString msg = evt->exevent->response->message;
            string key_word = "To: <sip:";
            int idx = msg.indexOf(key_word.c_str());
            if(idx >= 0)
            {
                strncpy(dest_id, msg.toStdString().c_str() + idx + key_word.length(), SIP_ID_LEN);
                cam_idx = getCamIndexByID(dest_id);
            }
        }

        stringstream ss;
        ss << "v=0\r\n";
        ss << "o=" << m_DeviceID.toStdString() << " 0 0 IN IP4 " << m_localIP.toStdString() << "\r\n";
        ss << "s=Play\r\n";
        ss << "c=IN IP4 " << m_localIP.toStdString() << "\r\n";
        ss << "t=0 0\r\n";
        if (rtp_protocol == "TCP/RTP/AVP")
        {
            ss << "m=video " << m_localPort.toStdString() << " TCP/RTP/AVP 96\r\n";
        }
        else
        {
            ss << "m=video " << m_localPort.toStdString() << " RTP/AVP 96\r\n";
        }
        ss << "a=sendonly\r\n";
        ss << "a=rtpmap:96 PS/90000\r\n";
        ss << "y=" << ssrc << "\r\n";
        string sdp_output_str = ss.str();

        size_t size = sdp_output_str.size();

        osip_message_t *message = evt->exevent->request;
        int status = eXosip_call_build_answer(m_excontext, evt->exevent->tid, 200, &message);

        if (status != 0)
        {
            //spdlog::error("call invite build answer failed");
            return -1;
        }

        osip_message_set_content_type(message, "APPLICATION/SDP");
        osip_message_set_body(message, sdp_output_str.c_str(), sdp_output_str.size());

        eXosip_call_send_answer(m_excontext, evt->exevent->tid, 200, message);

        notifyStartVideo(cam_idx, rtp_ip, rtp_port, ssrc);
        //spdlog::info("reply call invite: \n{}", sdp_output_str);
        return 0;
    }
}
