#include "CSipMessageHandle.h"
#include <QString>
#include <sstream>
#include <QDebug>

namespace GB28181
{
#define MSGPROC_TEMPLATE(F) (bind(&CSipMessageHandle::F, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3))

    CSipMessageHandle::CSipMessageHandle()
    {
        m_devctlproc.insert(make_pair(PTZCmd, MSGPROC_TEMPLATE(on_devctrl_ptzcmd)));
        m_devctlproc.insert(make_pair(TeleBoot, MSGPROC_TEMPLATE(on_devctrl_teleboot)));
        m_devctlproc.insert(make_pair(RecordCmd, MSGPROC_TEMPLATE(on_devctrl_recordcmd)));
        m_devctlproc.insert(make_pair(GuardCmd, MSGPROC_TEMPLATE(on_devctrl_guardcmd)));
        m_devctlproc.insert(make_pair(AlarmCmd, MSGPROC_TEMPLATE(on_devctrl_alarmcmd)));
        m_devctlproc.insert(make_pair(IFrameCmd, MSGPROC_TEMPLATE(on_devctrl_iframecmd)));
        m_devctlproc.insert(make_pair(DragZoomIn, MSGPROC_TEMPLATE(on_devctrl_dragzoomin)));
        m_devctlproc.insert(make_pair(DragZoomOut, MSGPROC_TEMPLATE(on_devctrl_dragzoomout)));
        m_devctlproc.insert(make_pair(HomePosition, MSGPROC_TEMPLATE(on_devctrl_homeposition)));

        m_devcfgproc.insert(make_pair(BasicParam, MSGPROC_TEMPLATE(on_devcfg_basicparam)));
        m_devcfgproc.insert(make_pair(SVACEncodeConfig, MSGPROC_TEMPLATE(on_devcfg_svacencodeconfig)));
        m_devcfgproc.insert(make_pair(SVACDecodeConfig, MSGPROC_TEMPLATE(on_devcfg_svacdeconfigconfig)));

        m_queryproc.insert(make_pair(MANSCDP_QUERY_CMD_ALARM, MSGPROC_TEMPLATE(on_query_alarm)));
        m_queryproc.insert(make_pair(MANSCDP_QUERY_CMD_CATALOG, MSGPROC_TEMPLATE(on_query_catalog)));
        m_queryproc.insert(make_pair(MANSCDP_QUERY_CMD_DEVICE_INFO, MSGPROC_TEMPLATE(on_query_device_info)));
        m_queryproc.insert(make_pair(MANSCDP_QUERY_CMD_RECORD_INFO, MSGPROC_TEMPLATE(on_query_record_info)));
        m_queryproc.insert(make_pair(MANSCDP_QUERY_CMD_DEVICE_STATUS, MSGPROC_TEMPLATE(on_query_device_status)));
        m_queryproc.insert(make_pair(MANSCDP_QUERY_CMD_CONFIG_DOWNLOAD, MSGPROC_TEMPLATE(on_query_config_download)));
        m_queryproc.insert(make_pair(MANSCDP_QUERY_CMD_PRESET_QUERY, MSGPROC_TEMPLATE(on_query_preset_query)));
        m_queryproc.insert(make_pair(MANSCDP_QUERY_CMD_MOBILE_POSITION, MSGPROC_TEMPLATE(on_query_mobile_position)));
    }

    int CSipMessageHandle::sendSimplyResp(const char *uname, eXosip_t *excontext, int tid, int status)
    {
        osip_message_t *answer = nullptr;

        eXosip_lock((eXosip_t *)excontext);
        eXosip_message_build_answer(excontext, tid, status, &answer);
        int r = eXosip_message_send_answer(excontext, tid, status, nullptr);
        eXosip_unlock(excontext);
        return r;
    }

    //    int CSipMessageHandle::sendCallAck(eXosip_t *excontext, int did)
    //    {
    //        osip_message_t *ack = nullptr;
    //        eXosip_lock(excontext);
    //        eXosip_call_build_ack(excontext, did, &ack);
    //        eXosip_call_send_ack(excontext, did, ack);
    //        eXosip_unlock(excontext);
    //        return 0;
    //    }

    int CSipMessageHandle::getStatcodeFromResp(osip_message_t *resp)
    {
        return resp != nullptr ? resp->status_code : -1;
    }

    const char *CSipMessageHandle::getMsgIdFromReq(osip_message_t *req)
    {
        osip_generic_param_t *tag = nullptr;
        osip_to_get_tag(req->from, &tag);
        if (nullptr == tag || nullptr == tag->gvalue)
        {
            return "";
        }
        return (const char *)tag->gvalue;
    }

    int CSipMessageHandle::procMessage(const SIP_ENENT_PTR &e)
    {
        const char *username = e->exevent->request->from->url->username;
        const char *host = e->exevent->request->from->url->host;

        osip_body_t *body = nullptr;
        osip_message_get_body(e->exevent->request, 0, &body);
        if (body == nullptr)
        {
            sendSimplyResp(username, e->excontext, e->exevent->tid, SIP_BAD_REQUEST);
            return -1;
        }
        int r;

        //LOG_DEBUG << "incoming request body: " << body->body;

        tinyxml_doc_t doc;
        r = m_xmlparser.Load(body->body, body->length, doc);
        if (r != 0)
        {
            sendSimplyResp(username, e->excontext, e->exevent->tid, SIP_BAD_REQUEST);
        }

        MANSCDP_MSG_HEADER bodyheader;
        r = m_xmlparser.ParseXmlHeader(doc, bodyheader);
        if (r != 0)
        {
            sendSimplyResp(username, e->excontext, e->exevent->tid, SIP_BAD_REQUEST);
            return r;
        }

        //LOG_DEBUG << "Revice message, category: " << bodyheader.cmd_category << ", cmd_type: "
        //<< bodyheader.cmd_type << " from " << username << ", host: " << host;

        switch (bodyheader.cmd_category)
        {
        case MANSCDP_CMD_CATEGORY_CONTROL:
            r = handle_incoming_req_control(e, doc, bodyheader);
            break;
        case MANSCDP_CMD_CATEGORY_QUERY:
            r = handle_incoming_req_query(e, doc, bodyheader);
            break;
        case MANSCDP_CMD_CATEGORY_NOTIFY:

            break;
        case MANSCDP_CMD_CATEGORY_RESPONSE:

            break;
        default:
            break;
        }
        return 0;
    }

    int CSipMessageHandle::respMessage(const SIP_ENENT_PTR &e)
    {
        int statcode = getStatcodeFromResp(e->exevent->response);
        string reqid = getMsgIdFromReq(e->exevent->request);

        onMsgResponse(reqid, statcode);
        return 0;
    }

    int CSipMessageHandle::HandleResponseFailure(const SIP_ENENT_PTR &e)
    {
        respMessage(e);
        return 0;
    }

    int CSipMessageHandle::handle_incoming_req_control(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc,
                                                       MANSCDP_MSG_HEADER &bh)
    {
        if (bh.cmd_type == MANSCDP_CONTROL_CMD_DEVICE_CONTROL)
        {
            for (auto cmd : bh.devctl_subcmd)
            {
                m_devctlproc[cmd](e, doc, bh);
            }
        }
        else if (bh.cmd_type == MANSCDP_CONTROL_CMD_DEVICE_CONFIG)
        {
            for (auto cmd : bh.devcfg_subcmd)
            {
                m_devcfgproc[cmd](e, doc, bh);
            }
        }
        return 0;
    }

    int CSipMessageHandle::handle_incoming_req_query(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc,
                                                     MANSCDP_MSG_HEADER &bh)
    {
        auto proc = m_queryproc.find(bh.cmd_type);
        if (proc == m_queryproc.end())
        {
            //LOG_WARN << "Not found proc, cmd_type: " << bh.cmd_type;
            sendSimplyResp(e->name, e->excontext, e->exevent->tid, SIP_BAD_REQUEST);
            return -1;
        }
        m_queryproc[bh.cmd_type](e, doc, bh);
        return 0;
    }

    int CSipMessageHandle::handle_incoming_req_notify(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc,
                                                      MANSCDP_MSG_HEADER &bh)
    {
        return 0;
    }

    int CSipMessageHandle::handle_incoming_req_response(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc,
                                                        MANSCDP_MSG_HEADER &bh)
    {
        return 0;
    }

    ///< DeviceControl REQUEST; Control action
    int CSipMessageHandle::on_devctrl_ptzcmd(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        const char *username = e->exevent->request->from->url->username;
        DEVICE_CONTROL_CMD ctrlcmd;
        memset(&ctrlcmd, 0x0, sizeof(ctrlcmd));

        string cmdstr;
        int r = m_xmlparser.ParseEleStr(doc.RootElement(), "PTZCmd", cmdstr, true);
        if (r != 0)
        {
            //LOG_WARN << "ParseEleStr PTZCMD failed, ret: " << r;
            sendSimplyResp(username, e->excontext, e->exevent->tid, SIP_BAD_REQUEST);
            return -1;
        }

        int statcode = SIP_OK;
        m_ptzparser.ParseControlCmd(ctrlcmd, cmdstr);

        r = HandlePTZControl(ctrlcmd, bh.devid);
        if (r != 0)
        {
            statcode = SIP_BAD_REQUEST;
        }
        sendSimplyResp(username, e->excontext, e->exevent->tid, statcode);
        return 0;
    }

    int CSipMessageHandle::on_devctrl_teleboot(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        devctrl_subcmd(e, bh.devid, TeleBoot, SWITCH_ON);
        return 0;
    }

    int CSipMessageHandle::on_devctrl_recordcmd(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        string value;
        m_xmlparser.ParseEleStr(doc.RootElement(), "RecordCmd", value, true);

        MANSCDP_SWITCH_STATUS onoff;
        if (value == "Record")
            onoff = SWITCH_ON;
        else ///< StopRecord
            onoff = SWITCH_OFF;
        devctrl_subcmd(e, bh.devid, RecordCmd, onoff);
        return 0;
    }

    int CSipMessageHandle::on_devctrl_guardcmd(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        string value;
        m_xmlparser.ParseEleStr(doc.RootElement(), "GuardCmd", value, true);

        MANSCDP_SWITCH_STATUS onoff;
        if (value == "SetGuard")
            onoff = SWITCH_ON;
        else ///< ResetGuard
            onoff = SWITCH_OFF;
        devctrl_subcmd(e, bh.devid, GuardCmd, onoff);
        return 0;
    }

    int CSipMessageHandle::on_devctrl_alarmcmd(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        devctrl_subcmd(e, bh.devid, AlarmCmd, SWITCH_ON);
        return 0;
    }

    int CSipMessageHandle::on_devctrl_iframecmd(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        devctrl_subcmd(e, bh.devid, IFrameCmd, SWITCH_ON);
        return 0;
    }

    int CSipMessageHandle::on_devctrl_dragzoomin(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        return 0;
    }

    int CSipMessageHandle::on_devctrl_dragzoomout(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        return 0;
    }

    int CSipMessageHandle::on_devctrl_homeposition(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        return 0;
    }

    ///< DeviceConfig QUERST; Control action
    int CSipMessageHandle::on_devcfg_basicparam(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        return 0;
    }

    int
    CSipMessageHandle::on_devcfg_svacencodeconfig(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        return 0;
    }

    int
    CSipMessageHandle::on_devcfg_svacdeconfigconfig(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        return 0;
    }

    int CSipMessageHandle::devctrl_subcmd(const SIP_ENENT_PTR &e, string &devid, MANSCDP_DEV_CTRL_CMD subcmd,
                                          MANSCDP_SWITCH_STATUS onoff)
    {
        ///< TODO follow-up work£¬ carry response;
        const char *username = e->exevent->request->from->url->username;

        int statcode = OSIP_SUCCESS;
        int r = -1;

        r = HandleDeviceControl(subcmd, devid, SWITCH_ON);
        if (r != 0)
        {
            statcode = SIP_BAD_REQUEST;
        }
        sendSimplyResp(username, e->excontext, e->exevent->tid, statcode);
        return 0;
    }

    ///< Query action
    int CSipMessageHandle::on_query_device_status(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        //        manscdp_device_status_dialog_t dialog;
        //        m_xmlparser.ParseEleStr(doc.RootElement(), "SN", dialog.request.sn, true);
        //        m_xmlparser.ParseEleStr(doc.RootElement(), "DeviceID", dialog.request.devid, true);

        stringstream ss;

        time_t rawtime;
        struct tm *timeinfo;
        time(&rawtime);
        timeinfo = localtime(&rawtime);
        char curtime[72] = {0};
        sprintf(curtime, "%d-%d-%dT%02d:%02d:%02d", (timeinfo->tm_year + 1900), (timeinfo->tm_mon + 1),
                timeinfo->tm_mday, timeinfo->tm_hour, timeinfo->tm_min, timeinfo->tm_sec);

        ss << "<?xml version=\"1.0\"?>\r\n";
        ss << "<Response>\r\n";
        ss << "<CmdType>DeviceStatus</CmdType>\r\n";
        ss << "<SN>" << bh.sn.c_str() << "</SN>\r\n";
        ss << "<DeviceID>" << m_DeviceID.toStdString() << "</DeviceID>\r\n";
        ss << "<Result>OK</Result>\r\n";
        ss << "<Online>ONLINE</Online>\r\n";
        ss << "<Status>OK</Status>\r\n";
        ss << "<DeviceTime>" << curtime << "</DeviceTime>\r\n";
        ss << "<Alarmstatus Num=\"0\">\r\n";
        ss << "</Alarmstatus>\r\n";
        ss << "<Encode>ON</Encode>\r\n";
        ss << "<Record>OFF</Record>\r\n";
        ss << "</Response>\r\n";

        return sendResponse(e->exevent, "Application/MANSCDP+xml", ss.str());
    }

    int CSipMessageHandle::on_query_catalog(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        ///< TODO follow-up work£¬ carry response;
        const char *username = e->exevent->request->from->url->username;
        int statcode = OSIP_SUCCESS;

        string catalog = getCatalog(bh.sn.c_str());
        return sendRequest("Application/MANSCDP+xml", catalog);
    }

    int CSipMessageHandle::on_query_device_info(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        stringstream ss;

        ss << "<?xml version=\"1.0\"?>\r\n";
        ss << "<Response>\r\n";
        ss << "<CmdType>DeviceInfo</CmdType>\r\n";
        ss << "<SN>" << bh.sn.c_str()  << "</SN>\r\n";
        ss << "<DeviceID>" << m_DeviceID.toStdString() << "</DeviceID>\r\n";
        ss << "<Result>OK</Result>\r\n";
        ss << "<DeviceType>simulate client</DeviceType>\r\n";
        ss << "<Manufacturer>ZHD</Manufacturer>\r\n";
        ss << "<Model>28181</Model>\r\n";
        ss << "<Firmware>fireware</Firmware>\r\n";
        ss << "<MaxCamera>1</MaxCamera>\r\n";
        ss << "<MaxAlarm>0</MaxAlarm>\r\n";
        ss << "</Response>\r\n";

        return sendResponse(e->exevent, "Application/MANSCDP+xml", ss.str());
    }

    int CSipMessageHandle::on_query_record_info(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        return 0;
    }

    int CSipMessageHandle::on_query_alarm(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        return 0;
    }

    int CSipMessageHandle::on_query_config_download(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        return 0;
    }

    int CSipMessageHandle::on_query_preset_query(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        return 0;
    }

    int CSipMessageHandle::on_query_mobile_position(const SIP_ENENT_PTR &e, tinyxml_doc_t &doc, MANSCDP_MSG_HEADER &bh)
    {
        return 0;
    }
}
