#include "CGPSManager.h"
#include "socket/TcpServer.h"
#include "base_module/factory.h"
#include "timer/CTimerManager.h"

#include "math.h"
#include <QDateTime>
#include "uart.h"
#include <QFile>
#include <unistd.h>


#define INVALID_FD -1
#define TEST_UART_STUB false

REGISTER_MODULE(CGPSManager, "CGPSManager")
LOG_DEFINITION(CGPSManager)

extern QString getMainMacAddress();

CGPSManager::CGPSManager(QObject *parent)
    : QObject(parent),
      IModule(),
      m_pTcpModule(nullptr),
      m_fd(INVALID_FD),
      m_totalMileage(0),
      m_totalStopTime(0),
      m_iGpsErrorTimes(0),
      m_aiStatus(0),
      last_ai_report_time(0),
      m_pdeviceMng(nullptr)
{
    memset(&m_lastGps, 0, sizeof(m_lastGps));
    LOG_CREATE_WITH_COUNT(CGPSManager, "gps", 100000);
}

CGPSManager::~CGPSManager()
{
    std::cout << name() << std::endl;
}

//初始化操作
bool CGPSManager::Initialize()
{
    //起主消息处理线程
    STARTUP_TASK(TaskMain)

    connect(this, SIGNAL(notifyOpenGpsAutoReoprt()), this, SLOT(InitGpsUART()));
    connect(&m_UartPort, SIGNAL(readyRead()), this, SLOT(onReadUartData()));
	m_pdeviceMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));

    emit notifyOpenGpsAutoReoprt();

    STARTUP_TASK(TaskMain);
    return true;
}

bool CGPSManager::RegistMessage()
{
    m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    if (m_pTcpModule)
    {
        m_pTcpModule->RegistMessageCallback(this, MODULE_SENSOR_MNG);
    }
	m_pTimerMng = dynamic_cast<CTimerManager *>(::QueryPlugin("CTimerManager"));
    if (m_pTimerMng)
    {
        m_pTimerMng->RegTimer(MODULE_SENSOR_MNG, TIMER_CHECK_AI_REPORT_TIMER, (5*60));
    	}
    return true;
}

bool CGPSManager::getCurGPSInfo(GPS_INFO &gps)
{
    gps = m_lastGps;
    // 做个判断,如果经纬度都是0即没有获取到合理的gps信息
    if (m_lastGps.latitude != 0 && m_lastGps.longitude != 0)
    {
        return true;
    }
    return false;
}

//
bool CGPSManager::openGpsReport()
{
    qint64 len = 0;
    QSerialPort uart;
    const char gps_cmd[] = "AT+QGPS=1\r\n";
    // open
    QFile file("/dev/ttyUSB3");
	if(!file.exists())
	{
		LOG_ERROR(CGPSManager, "/dev/ttyUSB3 NOT EXIT");
        mSleep(1000);
        return false;
	}
    if (openUart(uart, "/dev/ttyUSB3", QIODevice::ReadWrite))
    {
        len = uart.write(gps_cmd, strlen(gps_cmd));
        mSleep(100);
        QByteArray buf;
        while (uart.waitForBytesWritten())
        {
            qApp->processEvents();
            buf = uart.readAll();
        }
        buf.clear();
        uart.clear();
        uart.close();
    }

    if (len < strlen(gps_cmd))
    {
        return false;
    }

    return true;
}

// 初始化gps UART
bool CGPSManager::openUart(QSerialPort &uart, QString port, QIODevice::OpenMode mode)
{
    if (uart.isOpen())
    {
        uart.close();
    }

    uart.setPortName(port);
    //打开串口

    if (!uart.open(mode))
    {
        LOG_INFO(CGPSManager, "m_UartPort open failed;");
        return false;
    }

    //设置波特率
    if (!uart.setBaudRate(19200))
    {
        LOG_INFO(CGPSManager, "setBaudRate failed;");
        return false;
    }
    //设置校验位
    if (!uart.setParity(QSerialPort::NoParity))
    {
        LOG_INFO(CGPSManager, "setParity failed;");
        return false;
    }

    //设置数据位
    if (!uart.setDataBits(QSerialPort::Data8))
    {
        LOG_INFO(CGPSManager, "setDataBits failed;");
        return false;
    }

    //设置停止位
    if (!uart.setStopBits(QSerialPort::OneStop))
    {
        LOG_INFO(CGPSManager, "setStopBits failed;");
        return false;
    }
    //设置流控制
    if (!uart.setFlowControl(QSerialPort::NoFlowControl))
    {
        LOG_INFO(CGPSManager, "setFlowControl failed;");
        return false;
    }

    return true;
}

// 初始化gps UART
void CGPSManager::InitGpsUART()
{
    if (TEST_UART_STUB)
    {
        return;
    }
    openGpsReport();
    openGpsUart();

    LOG_INFO(CGPSManager, "InitGpsUART");
}

void CGPSManager::openGpsUart()
{
    bool bSuccess = false;
    int index = 0;
    while (index++ < 3 && bSuccess == false)
    {
        //初始化串口链接
        QFile file("/dev/ttyUSB1");
    	if(!file.exists())
    	{
			LOG_ERROR(CGPSManager, "/dev/ttyUSB1 NOT EXIT");
			m_fd = -1;
            mSleep(1000);
            continue;
		}
        m_fd = UART_Open("/dev/ttyUSB1");
        if (m_fd == -1)
        {
            LOG_ERROR(CGPSManager, "UART_Openfail");
            mSleep(1000);
            continue;
        }

        if (0 != UART_Init(m_fd))
        {
            UART_Close(m_fd);
            m_fd = 0;
            LOG_ERROR(CGPSManager, "UART_Init");
            mSleep(1000);
            continue;
        }

        //        if(!openUart(m_UartPort,  "/dev/ttyUSB1", QIODevice::ReadOnly))
        //        {
        //            mSleep(1000);
        //            continue;
        //        }

        bSuccess = true;
        break;
    }

    LOG_INFO(CGPSManager, "openGpsUart Result:%d;", (int)bSuccess);
    return;
}

bool CGPSManager::readUart(QString &info)
{
    QByteArray buf;
    if (!m_UartPort.isOpen())
    {
        emit notifyOpenGpsAutoReoprt();
        mSleep(1000);
        return false;
    }

    buf = m_UartPort.readAll();

    if (!buf.isEmpty())
    {

        info = QString(buf);
    }
    buf.clear();
    return true;
}

void CGPSManager::onReadUartData()
{
    qDebug("onReadUartData");
    QString gps_string;

    readUart(gps_string);

    if (gps_string.isEmpty())
    {
        return;
    }

    procGpsData(gps_string);
}

// 关闭gps uart
bool CGPSManager::closeGpsUART()
{
    if (TEST_UART_STUB)
    {
        return true;
    }

    disconnect(&m_UartPort, SIGNAL(readyRead()), this, SLOT(ReadUartData()));
    //关闭串口
    m_UartPort.clear();
    m_UartPort.close();
}

// 使能gps定位
bool CGPSManager::enableGPSLOC()
{
    if (TEST_UART_STUB)
    {
        return true;
    }

    emit notifyOpenGpsAutoReoprt();

    return true;
}

bool CGPSManager::parseGpsQGPSLoc(QString str_info, GPS_INFO &info)
{
    bool bSuccess = false;
    // qDebug() << rev_buf;
    // prase gps info
    info.bValid = false;
    int idx = str_info.indexOf("QGPSLOC:");
    const int offset = strlen("QGPSLOC:");
    if (idx >= 0)
    {
        str_info = str_info.mid(idx + offset);
        QStringList str_list = str_info.split(',');
        // GPSLOC Format time, lau, long, hdop, high, fix, cog, spkx,spkn, date, satn
        // QGPSLOC: 034719.0,3034.5297N,10354.9545E,0.7,505.0,2,160.41,0.0,0.0,060821,07
        if (str_list.length() == 11)
        {
            info.bValid = true;
            QDateTime tm;
            int dot_idx = str_list[0].lastIndexOf('.');
            QString str_tm = str_list[0].left(dot_idx);
            str_tm = str_tm.replace(" ", "");
            QTime t = QTime::fromString(str_tm, "hhmmss");
            QDate d = QDate::fromString(str_list[9], "ddMMyy");

            // 19xx -> 20xx
            d.setDate(d.year() + 100, d.month(), d.day());

            tm.setTime(t);
            tm.setDate(d);
            // utc->local +8:00
            tm.setTime_t(tm.toTime_t() + 28800);

            info.time_tick = tm.toTime_t();
            QString strlat = str_list[1];
            strlat.replace("N", "");
            strlat.replace("S", "");
            info.latitude = strlat.toDouble();
            QString strlong = str_list[2];
            strlong.replace("E", "");
            strlong.replace("W", "");
            info.longitude = strlong.toDouble();
            info.bEast = true;
            info.bNorth = true;

            info.hdop = str_list[3].toFloat();
            info.high = str_list[4].toFloat();
            info.dir = str_list[6].toFloat();
            info.speed = str_list[7].toFloat();
            info.sat_num = str_list[10].toInt();
            bSuccess = true;
        }
    }
    else
    {
        // prase error
        praseUartError(str_info);
    }

    return bSuccess;
}

// $GPGSV,4,2,14,21,13,274,40,22,21,314,33,23,09,150,27,25,37,057,33*73
// $GPGSV,4,3,14,26,28,188,25,29,,,30,31,64,284,32,32,58,042,28*44
// $GPGSV,4,4,14,39,,,36,41,,,36*73
// $GPGGA,103054.00,3033.145155,N,10403.784055,E,1,08,0.9,706.4,M,-31.0,M,,*4B
// $GPVTG,0.0,T,1.9,M,0.0,N,0.0,K,A*2B
// $GPRMC,103054.00,A,3033.145155,N,10403.784055,E,0.0,0.0,280921,1.9,W,A*23
// $GPGSA,A,2,10,12,21,22,23,25,31,32,,,,,1.2,0.9,0.8*35
// $GPGSV,4,1,14,01,12,293,24,03,02,322,38,10,41,158,24,12,12,039,25*7F
// $GPGSV,4,2,14,21,13,274,40,22,21,314,34,23,09,150,27,25,37,057,34*73
// $GPGSV,4,3,14,26,28,188,26,29,,,31,31,64,284,32,32,58,042,28*46
// $GPGSV,4,4,14,39,,,36,41,,,36*73
// $GPGGA,103054.00,3033.129361,N,10403.788391,E,1,09,0.7,703.8,M,-31.0,M,,*45
// $GPVTG,0.0,T,1.9,M,0.0,N,0.0,K,A*2B
// $GPRMC,103054.00,A,3033.129361,N,10403.788391,E,0.0,0.0,280921,1.9,W,A*2B
// $GPGSA,A,2,01,10,12,21,22,23,25,31,32,,,,1.1,0.7,0.8*39
// $GPGSV,4,1,14,01,12,293,24,03,02,322,37,10,41,158,24,12,12,039,26*73
// $GPGSV,4,2,14,21,13,274,40,22,21,314,34,23,09,150,26,25,37,057,32*74
// $GPGSV,4,3,14,26,28,188,27,29,,,32,31,64,284,32,32,58,042,26*4A
// $GPGSV,4,4,14,39,,,36,41,,,36*73
// $GPGGA,103055.00,3033.129900,N,10403.788310,E,1,09,0.7,703.8,M,-31.0,M,,*40
// $GPVTG,0.0,T,1.9,M,0.0,N,0.0,K,A*2B
// $GPRMC,103055.00,A,3033.129900,N,10403.788310,E,0.0,0.0,280921,1.9,W,A*2E

bool CGPSManager::parseGpsGGA(QString str_info, GPS_INFO &info)
{
    //    str_info = "$GPGGA,103055.00,3033.129900,N,10403.788310,E,1,09,0.7,703.8,M,-31.0,M,,*40";
    bool bSuccess = false;
    //    qDebug() << str_info;
    int idx = str_info.indexOf("GPGGA");
    const int offset = idx + strlen("GPGGA");
    if (idx >= 0)
    {
        //$GPGGA 语句包括17个字段：语句标识头，世界时间，纬度，纬度半球，经度，经度半球，定位质量指示，使用卫星数量，水平精确度，海拔高度，高度单位，大地水准面高度，高度单位，差分GPS数据期限，差分参考基站标号，校验和结束标记(用回车符<CR>和换行符<LF>)，分别用14个逗号进行分隔。该数据帧的结构及各字段释义如下：

        //$GPGGA,<1>,<2>,<3>,<4>,<5>,<6>,<7>,<8>,<9>,M,<10>,M,<11>,<12>*xx<CR><LF>
        // $GPGGA,103055.00,3033.129900,N,10403.788310,E,1,09,0.7,703.8,M,-31.0,M,,*40
        str_info = str_info.mid(idx + offset);
        QStringList str_list = str_info.split(',');
        if (str_list.length() == 14)
        {
            if (info.latitude == 0)
            {
                QString strlat = str_list[1];
                strlat.replace("N", "");
                strlat.replace("S", "");
                info.latitude = strlat.toDouble();
                info.bNorth = true;
            }

            if (info.longitude == 0)
            {
                QString strlong = str_list[3];
                strlong.replace("E", "");
                strlong.replace("W", "");
                info.longitude = strlong.toDouble();
                info.bEast = true;
            }

            info.sat_num = str_list[6].toFloat();
            info.high = str_list[8].toFloat();
            bSuccess = true;
        }
    }
    else
    {
        // prase error
        praseUartError(str_info);
    }

    return bSuccess;
}

bool CGPSManager::parseGpsRMC(QString str_info, GPS_INFO &info)
{
    //    str_info = "$GPRMC,103055.00,A,3033.129900,N,10403.788310,E,0.0,0.0,280921,1.9,W,A*2E";
    bool bSuccess = false;
    LOG_INFO(CGPSManager, "%s", str_info.toStdString().c_str());
    //    qDebug() << str_info;
    info.bValid = false;
    int idx = str_info.indexOf("GPRMC");
    const int offset = idx + strlen("GPRMC");
    if (idx >= 0)
    {
        //$GPRMC,<1>,<2>,<3>,<4>,<5>,<6>,<7>,<8>,<9>,<10>,<11>,<12>*hh
        //<1> UTC时间，hhmmss(时分秒)格式 //好像还有毫秒。
        //<2> 定位状态，A=有效定位，V=无效定位
        //<3> 纬度ddmm.mmmm(度分)格式(前面的0也将被传输)
        //<4> 纬度半球N(北半球)或S(南半球)
        //<5> 经度dddmm.mmmm(度分)格式(前面的0也将被传输)
        //<6> 经度半球E(东经)或W(西经)
        //<7> 地面速率(000.0~999.9节，前面的0也将被传输)
        //<8> 地面航向(000.0~359.9度，以真北为参考基准，前面的0也将被传输)
        //<9> UTC日期，ddmmyy(日月年)格式
        //<10> 磁偏角(000.0~180.0度，前面的0也将被传输)
        //<11> 磁偏角方向，E(东)或W(西)
        //<12> 模式指示(仅NMEA0183 3.00版本输出，A=自主定位，D=差分，E=估算，N=数据无效)
        str_info = str_info.mid(idx + offset);
        QStringList str_list = str_info.split(',');
        // QGPSLOC: 034719.0,A,3034.5297,N,10354.9545,E,0.7,505.0,2,160.41,0.0,0.0,060821,07
        if (str_list.length() >= 12)
        {
            info.bValid = true;
            QDateTime tm;
            int dot_idx = str_list[0].lastIndexOf('.');
            QString str_tm = str_list[0].left(dot_idx);
            str_tm = str_tm.replace(" ", "");
            QTime t = QTime::fromString(str_tm, "hhmmss");
            QDate d = QDate::fromString(str_list[8], "ddMMyy");

            // 19xx -> 20xx
            d.setDate(d.year() + 100, d.month(), d.day());

            tm.setTime(t);
            tm.setDate(d);
            // utc->local +8:00
            tm.setTime_t(tm.toTime_t() + 28800);

            info.time_tick = tm.toTime_t();
            QString strlat = str_list[2];
            strlat.replace("N", "");
            strlat.replace("S", "");
            info.latitude = strlat.toDouble();
            QString strlong = str_list[4];
            strlong.replace("E", "");
            strlong.replace("W", "");
            info.longitude = strlong.toDouble();
            info.bEast = true;
            info.bNorth = true;

            info.dir = str_list[7].toFloat();
            info.speed = str_list[6].toFloat() * 1.852;
            info.bValid = str_list[1].indexOf("A") > -1 ? true : false;
            bSuccess = true;
        }
    }
    else
    {
        // prase error
        praseUartError(str_info);
    }

    return bSuccess;
}

QString testGenGpsInfo()
{
    static int tm_tick = 0;
    tm_tick++;
    if (tm_tick == 400)
    {
        tm_tick = 0;
    }

    int flag = tm_tick / 100;

    static double lat = 3034.5297;
    static double lon = 10354.9545;

    switch (flag)
    {
    case 0:
        lat += 0.1;
        break;
    case 1:
        lon += 0.1;
        break;
    case 2:
        lat -= 0.1;
        break;
    case 3:
        lon -= 0.1;
        break;
    default:
        break;
    }
    QDate date = QDate::currentDate();
    QTime time = QTime::currentTime();
    //    QString str_gps = QString("QGPSLOC: %3.0,%1N,%2E,0.7,505.0,2,160.41,66.6,0.0,%4,07")
    //            .arg(lat, 10, 10, 'g', 5)
    //            .arg(lon, 10, 10, 'g', 5)
    //            .arg(time.toString("hhmmss"))
    //            .arg(date.toString("ddMMyy"));

    QString str_gps = QString().sprintf("QGPSLOC: %s.0,%.06lfN,%.06lfE,0.7,505.0,2,160.41,66.6,0.0,%s,07", time.toString("hhmmss").toStdString().c_str(), lat, lon, date.toString("ddMMyy").toStdString().c_str());

    // QString str_gps = "QGPSLOC: 034719.0,3034.5297N,10354.9545E,0.7,505.0,2,160.41,66.6,0.0," + date.toString("ddMMyy") + ",07";
    return str_gps;
}

//获取GPS信息
QString CGPSManager::getGPSinfo()
{
}

// 错误码处理
void CGPSManager::praseUartError(QString str_info)
{
    int idx = str_info.indexOf("CME ERROR:");
    const int offset = strlen("CME ERROR:");
    if (idx >= 0)
    {
        int ecode = str_info.mid(idx + offset, 3).toInt();
        switch (ecode)
        {
        // 501 Invalid parameter(s) 无效参数
        // 502 Operation not supported 操作不支持
        // 503 GNSS subsystem busy GNSS 子系统繁忙
        // 504 Session is ongoing 会话仍在进行
        // 506 Operation timeout 操作超时
        // 508 Time information error 时间信息错误
        // 512 Validity time is out of range 有效期过期
        // 513 Internal resource error 内部资源错误
        // 514 GNSS locked GNSS 锁住
        // 515 End by E911 由 E911 结束
        // 516 Not fixed now 当前未定位
        case 501:
        case 502:
        case 503:
        case 504:
        case 506:
        case 508:
        case 512:
        case 513:
        case 514:
        case 515:
        case 516:
            break;
        // 505 Session not active 会话未激活
        case 505:
            enableGPSLOC();
            break;
        // 507 Function not enabled 功能未使能
        case 507:
            enableGPSLOC();
            break;
        default:
            break;
        }
    }
    else
    {
    }
}

//主消息任务，主要用于和其他模块之间的消息通讯
void CGPSManager::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskSensorMain");
    MODULE_MESSAGE *pMsg = nullptr;
    RegistMessage();
    waitForStart();
    //时能GPS LOC信息
    enableGPSLOC();
    //起主消息处理线程
    STARTUP_TASK(TaskGPSCol)

    while (!isShutdown())
    {
        if (GetModuleMessage(pMsg)) // get msg from message queue
        {
            switch (pMsg->iCmd)
            {
             case CMD_SENSOR_MNG_STATUS:
                 ProcSTATUSModeRequest(*pMsg);
                 break;
			 case TIMER_CHECK_AI_REPORT_TIMER :
			 	ProcaiTimerRequest(*pMsg);
            default:
                break;
            }

            SAFE_DELETE(pMsg);
        }
    }
}


void CGPSManager::ProcSTATUSModeRequest(MODULE_MESSAGE &msg)
{

	printf("ProcSensor Status\n");
	last_ai_report_time = time(NULL);
	memcpy(&m_aiStatus, msg.szBuff, sizeof(int));
	emit notifyaiStatus(m_aiStatus);
	printf("----m_aiStatus %d ------\n", m_aiStatus);



}
void executeCMD(const char *cmd, char *result)   
{   
    char buf_ps[1024];   
    char ps[1024]={0};   
    FILE *ptr;   
    strcpy(ps, cmd);   
    if((ptr=popen(ps, "r"))!=NULL)   
    {   
        while(fgets(buf_ps, 1024, ptr)!=NULL)   
        {   
           strcat(result, buf_ps);   
           if(strlen(result)>1024)   
               break;   
        }   
        pclose(ptr);   
        ptr = NULL;  
		
    }   
    else  
    {   
        printf("popen %s error\n", ps);   
    }   
}  


void CGPSManager::ProcaiTimerRequest(MODULE_MESSAGE &msg)
{
	char error[1024];
	printf("ai timer Status\n");
	if (0 == last_ai_report_time)
	{
		printf("from start no report\n");
		executeCMD("docker restart ai-project", error);
		printf("error is %s len is %d\n", error, strlen(error));
	}
	time_t now = time(NULL);
	if((now - last_ai_report_time) >(5*60))
	{
		printf("ai report timer too long");
		executeCMD("docker restart ai-project", error);	
		printf("error is %s len is %d\n", error, strlen(error));
	}
}


bool CGPSManager::isValidGps(GPS_INFO &gps_info)
{
    // 检查gps标记
    if (!gps_info.bValid)
    {
        return false;
    }

    // 检查GPS时间
    QDateTime cur_time = QDateTime::currentDateTime();
    QDateTime gps_time = QDateTime::fromTime_t(gps_info.time_tick);
    int day_dis = abs(cur_time.daysTo(gps_time));
    if (day_dis > 100)
    {
        return false;
    }

    // 检查GPS位置信息
    if (gps_info.latitude == 0 && gps_info.longitude == 0)
    {
        return false;
    }

    return true;
}

bool CGPSManager::isLegal()
{
	return isValidGps(m_lastGps);
}

// 获取速度信息
float CGPSManager::getSpeedInfo()
{
    return m_lastGps.speed;
}

float CGPSManager::getAngleInfo()
{
    return m_lastGps.dir;
}

// 获取速度信息
uint CGPSManager::getMileageInfo()
{
    return (uint)m_totalMileage;
}

void CGPSManager::queryGpsByUart()
{
    if (TEST_UART_STUB)
    {
        return;
    }

    char rev_buf[1024];
    int data_len = 0;
    memset(rev_buf, 0, 1024);
    if (m_fd != -1)
    	data_len = UART_Recv(m_fd, rev_buf, sizeof(rev_buf) - 1);
	else 
		data_len = -1;
    if (data_len == -1 || data_len == 0)
    {
        emit notifyOpenGpsAutoReoprt();
		sleep(20);
    }

    QString gps_info = rev_buf;

    procGpsData(gps_info);
    //    const char gps_cmd[] = "AT+QGPSLOC=0\r";

    //    if(m_UartPort.write(gps_cmd, strlen(gps_cmd)) <= 0)
    //    {
    //        emit notifyOpenGpsAutoReoprt();
    //        mSleep(1000);
    //        return;
    //    }
}

//传感器收集任务, 用于定时轮询传感器相关的信息;
void CGPSManager::TaskGPSCol(void *pBuff)
{
    while (this->isShutdown() == false)
    {
        if (TEST_UART_STUB) //
        {
            QString gps_string = testGenGpsInfo();
            procGpsData(gps_string);
        }
        else
        {
            //轮询sensor信息
            queryGpsByUart();
        }

        // m_iGpsErrorTimes在收到正确的gps信息的情况下会清0
        if (m_iGpsErrorTimes++ > 60)
        {
            emit notifyGpsMasked(true);
        }
        else
        {
            emit notifyGpsMasked(false);
        }
        mSleep(1000);
    }
}

QString CGPSManager::getLatitude()
{
	QString strGPSInfo = QString().sprintf("%.08lf", m_lastGps.latitude);
	return strGPSInfo;
}
QString CGPSManager::getLongitude()
{
	QString strGPSInfo = QString().sprintf("%.08lf", m_lastGps.longitude);
	return strGPSInfo;
}

void CGPSManager::procGpsData(QString &gps_string)
{
    //
    QStringList lst = gps_string.split("\r\n");
    GPS_INFO gps_info;
    memset(&gps_info, 0, sizeof(gps_info));

    foreach (QString item, lst)
    {
        if (item.indexOf("GPRMC") >= 0)
        {
            parseGpsRMC(item, gps_info);
        }
        else if (item.indexOf("QGPSLOC") >= 0)
        {
            parseGpsQGPSLoc(item, gps_info);
        }
        else if (item.indexOf("GPGGA") >= 0)
        {
            parseGpsGGA(item, gps_info);
        }
    }

    if (!isValidGps(gps_info))
    {
    	QString strDevID = "12345678";
		if (m_pdeviceMng)
	{
		strDevID = m_pdeviceMng->getDeviceID();
	}
    	LOG_INFO(CGPSManager, "%s%s", "id Boardcast=>", strDevID.toStdString().c_str());
    	LOG_INFO(CGPSManager, "%s", "no Boardcast");
        return;
    }

    if (gps_info.bValid)
    {
        float distance = getDistanceByGPS(m_lastGps, gps_info);
        m_totalMileage += min(distance, gps_info.speed * 1000 / 3600);
        m_lastGps = gps_info;
    }
    else
		
    {
    	LOG_INFO(CGPSManager, "%s", "no Boardcast 11");
        return;
    }

    float speed = getSpeedInfo();
	float angle = getAngleInfo();

    if ((int)getSpeedInfo() == 0)
    {
        m_totalStopTime++;
        if (m_totalStopTime > 60)
        {
            emit notifySpeedAlarm(true);
        }
    }
    else
    {
        emit notifySpeedAlarm(false);
        m_totalStopTime = 0;
        m_iGpsErrorTimes = 0;
    }

    //给应用层发送消息,同步传感器信息
    MODULE_MESSAGE msg;
    msg.dst_id = MODULE_AI_SENSORS;
    msg.dst_type = APP_TYPE_AI;
    msg.iCmd = CMD_UPDATE_SENSOR_INFO;
    int idx = 0;
    QString strDevID = "12345678";
	if (m_pdeviceMng)
	{
		strDevID = m_pdeviceMng->getDeviceID();
	}
    QString strGPSInfo = QString().sprintf("[%.08lf,%.08lf]", m_lastGps.latitude, m_lastGps.longitude);
	printf("id : %s\n",strDevID.toStdString().c_str());
	LOG_INFO(CGPSManager, "%s%s", "id Boardcast=>", strDevID.toStdString().c_str());
    LOG_INFO(CGPSManager, "%s%s", "GPS Boardcast=>", strGPSInfo.toStdString().c_str());
    memcpy(msg.szBuff, strDevID.toStdString().c_str(), strDevID.length());
    idx += 64;
    memcpy(msg.szBuff + idx, strGPSInfo.toStdString().c_str(), strGPSInfo.length());
    idx += MAX_PATH;
    memcpy(msg.szBuff + idx, &speed, sizeof(speed));
    idx += sizeof(speed);
	memcpy(msg.szBuff + idx, &angle, sizeof(angle));
    printf("angle: %f\n", angle);
    printf("sizeof is %d\n",sizeof(msg));
    //模块间发送消息
    m_pTcpModule->SendMsg(msg, sizeof(msg));
    //界面emit消息
    Q_EMIT notifySensorUpdate();
}
