#include "uart.h"
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <iostream>
#include <unistd.h>
#include <QDateTime>
using namespace std;

//fd:串口文件描述符   port:串口号(/SERIAL1/SERIAL3)
int UART_Open(const char *port, int flag)
{
    int fd = open(port, flag);
    if (fd < 0)
    {
        perror("Can't Open Serial Port:");
        return -1;
    }

    //恢复串口为阻塞状态
    if (fcntl(fd, F_SETFL, 0) < 0)
    {
        printf("fcntl failed!\n");
        return -1;
    }
    else
    {
        printf("fcntl=%d\n", fcntl(fd, F_SETFL, 0));
    }
    printf("fd->open=%d\n", fd);
    return fd;
}

void UART_Close(int fd)
{
    close(fd);
}
/*******************************************************************
*名称：             UART_Set
*功能：             设置串口数据位，停止位和效验位
*入口参数：        fd          串口文件描述符
*                   speed       串口速度
*                   flow_ctrl   数据流控制
*                   databits    数据位   取值为7或者8
*                   stopbits    停止位   取值为1或者2
*                   parity      效验类型 取值为N,E,O,S
*******************************************************************/
int UART_Set(int fd, int speed, int flow_ctrl, int databits, int stopbits, char parity)
{
    int i;
    int status;
    int speed_arr[] = {B115200, B19200, B9600, B4800, B2400, B1200, B300};
    int name_arr[] = {115200, 19200, 9600, 4800, 2400, 1200, 300};

    struct termios options;

    /*tcgetattr(fd,&options)得到与fd指向对象的相关参数，并将它们保存于options,该函数还可以测试配置是否正确，
  该串口是否可用等。若调用成功，函数返回值为0，若调用失败，函数返回值为1.  */
    if (tcgetattr(fd, &options) != 0)
    {
        perror("SetupSerial 1");
        return -1;
    }

    //设置串口输入波特率和输出波特率
    for (i = 0; i < sizeof(speed_arr) / sizeof(int); i++)
    {
        if (speed == name_arr[i])
        {
            cfsetispeed(&options, speed_arr[i]);
            cfsetospeed(&options, speed_arr[i]);
        }
    }

    //修改控制模式，保证程序不会占用串口
    options.c_cflag |= CLOCAL;
    //修改控制模式，使得能够从串口中读取输入数据
    options.c_cflag |= CREAD;

    //设置数据流控制
    switch (flow_ctrl)
    {

    case 0: //不使用流控制
        options.c_cflag &= ~CRTSCTS;
        break;
    case 1: //使用硬件流控制
        options.c_cflag |= CRTSCTS;
        break;
    case 2: //使用软件流控制
        options.c_cflag |= IXON | IXOFF | IXANY;
        break;
    }
    //设置数据位
    //屏蔽其他标志位
    options.c_cflag &= ~CSIZE;
    switch (databits)
    {
    case 5:
        options.c_cflag |= CS5;
        break;
    case 6:
        options.c_cflag |= CS6;
        break;
    case 7:
        options.c_cflag |= CS7;
        break;
    case 8:
        options.c_cflag |= CS8;
        break;
    default:
        fprintf(stderr, "Unsupported data size\n");
        return -1;
    }
    //设置校验位
    switch (parity)
    {
    case 'n':
    case 'N': //无奇偶校验位。
        options.c_cflag &= ~PARENB;
        options.c_iflag &= ~INPCK;
        break;
    case 'o':
    case 'O': //设置为奇校验
        options.c_cflag |= (PARODD | PARENB);
        options.c_iflag |= INPCK;
        break;
    case 'e':
    case 'E': //设置为偶校验
        options.c_cflag |= PARENB;
        options.c_cflag &= ~PARODD;
        options.c_iflag |= INPCK;
        break;
    case 's':
    case 'S': //设置为空格
        options.c_cflag &= ~PARENB;
        options.c_cflag &= ~CSTOPB;
        break;
    default:
        fprintf(stderr, "Unsupported parity\n");
        return -1;
    }
    // 设置停止位
    switch (stopbits)
    {
    case 1:
        options.c_cflag &= ~CSTOPB;
        break;
    case 2:
        options.c_cflag |= CSTOPB;
        break;
    default:
        fprintf(stderr, "Unsupported stop bits\n");
        return -1;
    }

    //修改输出模式，原始数据输出
    options.c_oflag &= ~OPOST;

    options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
    //options.c_lflag &= ~(ISIG | ICANON);

    //设置等待时间和最小接收字符
    options.c_cc[VTIME] = 1; /* 读取一个字符等待1*(1/10)s */
    options.c_cc[VMIN] = 1;  /* 读取字符的最少个数为1 */

    //如果发生数据溢出，接收数据，但是不再读取 刷新收到的数据但是不读
    tcflush(fd, TCIFLUSH);

    //激活配置 (将修改后的termios数据设置到串口中）
    if (tcsetattr(fd, TCSANOW, &options) != 0)
    {
        perror("com set error!\n");
        return -1;
    }
    return 0;
}

/*******************************************************************
*名称：             UART_Init
*功能：             串口初始化
*入口参数：        fd          串口文件描述符
*                   speed       串口速度
*                   flow_ctrl   数据流控制
*                   databits    数据位   取值为7或者8
*                   stopbits    停止位   取值为1或者2
*                   parity      效验类型 取值为N,E,O,S
*******************************************************************/
int UART_Init(int fd, int speed, int flow_ctrl, int databits, int stopbits, char parity)
{
    if (UART_Set(fd, speed, 0, 8, 1, 'N') == -1)
        return -1;
    else
        return 0;
}

/*******************************************************************
* 名称：            UART0_Recv
* 功能：            接收串口数据
* 入口参数：       fd         文件描述符
*                   rcv_buf    接收串口中数据存入rcv_buf缓冲区中
*                   data_len   一帧数据的长度
*******************************************************************/
int UART_Recv(int fd, char *rcv_buf, int data_len)
{
    int len, fs_sel;
    fd_set fs_read;

    struct timeval time;

    FD_ZERO(&fs_read);
    FD_SET(fd, &fs_read);

    time.tv_sec = 10;
    time.tv_usec = 0;

    //使用select实现串口的多路通信
//    QTime tm = QTime::currentTime();
//    QString str_time = tm.toString("hh:mm:ss");
//    printf("time: %s :fd = %d\n",(char*)str_time.toStdString().c_str(),fd);

    len = read(fd, rcv_buf, data_len);

//    tm = QTime::currentTime();
//    str_time = tm.toString("hh:mm:ss");
//    printf("%s :read len = %d\n",(char*)str_time.toStdString().c_str(),len);
    return len;
    // fs_sel = select(fd + 1, &fs_read, NULL, NULL,&time);
    // tm = QTime::currentTime();
    // printf("%s :fd = %d, fs_sel = %d\n", tm.toString("hh:mm:ss").toStdString().c_str(),fd, fs_sel);
    // if (fs_sel)
    // {
    //     len = read(fd, rcv_buf, data_len);
    //     return len;
    // }
    // else
        // return -1;
}

/********************************************************************
* 名称：            UART0_Send
* 功能：            发送数据
* 入口参数：        fd           文件描述符
*                   send_buf     存放串口发送数据
*                   data_len     一帧数据的个数
*******************************************************************/
int UART_Send(int fd, const unsigned char *send_buf, int data_len)
{
    int len = 0;

    len = write(fd, send_buf, data_len);
    if (len == data_len)
    {
        // printf("send data is %s\n", send_buf);
        return len;
    }
    else
    {

        tcflush(fd, TCOFLUSH);
        return -1;
    }
}

unsigned short CRC16(unsigned char *p, unsigned short datalen)
{
    unsigned char CRC16Lo, CRC16Hi, CL, CH, SaveHi, SaveLo;
    unsigned short i, Flag;
    CRC16Lo = 0xFF;
    CRC16Hi = 0xFF;
    CL = 0x01;
    CH = 0xA0;

    for (i = 0; i < datalen; i++)
    {
        CRC16Lo ^= *(p + i); //每一个数据与CRC寄存器进行异或
        for (Flag = 0; Flag < 8; Flag++)
        {
            SaveHi = CRC16Hi;
            SaveLo = CRC16Lo;
            CRC16Hi >>= 1;
            CRC16Lo >>= 1;               //高位右移一位，低位右移一位
            if ((SaveHi & 0x01) == 0x01) //如果高位字节最后一位为1
                CRC16Lo |= 0x80;         //则低位字节右移后前面补1否则自动补0
            if ((SaveLo & 0x01) == 0x01) //如果LSB为1，则与多项式码进行异或
            {
                CRC16Hi ^= CH;
                CRC16Lo ^= CL;
            }
        }
    }
    return (unsigned short)(CRC16Hi << 8) | CRC16Lo;
}
