#ifndef CJT808ALARM_H
#define CJT808ALARM_H
#include "jt808_common.h"
#include "jt808_msg.h"
#include "common.h"
#include <string>
#include <vector>
#include <QMap>
#include <QRect>
#include <QPolygon>
#include <QPoint>

enum AreaOperation
{
    kUpdateArea = 0x0,
    kAppendArea,
    kModefyArea,
};

enum AreaRouteType
{
    kCircular = 0x1,
    kRectangle,
    kPolygonal,
    kRoute,
};

#pragma pack(push, 1)

// 区域属性定义
union AreaAttribute
{
    struct Bit
    {
        uint16_t bytime : 1;              // 1: 是否启用时间判断 否(0),是(1)
        uint16_t speedlimit : 1;          // 1: 十分启用速度判断 否(0),是(1)
        uint16_t inalarmtodirver : 1;     // 1: 进区域报警给驾驶员 否(0),是(1)
        uint16_t inalarmtoserver : 1;     // 1: 进区域报警给平台 否(0),是(1)
        uint16_t outalarmtodirver : 1;    // 1: 出区域报警给驾驶员 否(0),是(1)
        uint16_t outalarmtoserver : 1;    // 1: 出区域报警给平台 否(0),是(1)
        uint16_t snlatitude : 1;          // 0: 北纬:1; 1: 南纬
        uint16_t ewlongitude : 1;         // 0: 东经:1; 1: 西经
        uint16_t canopendoor : 1;         // 0: 允许开门:1; 1: 禁止开门
        uint16_t retain : 5;              // 保留
        uint16_t inopencommunication : 1; // 0: 进区域开启通信模块:1;
                                          // 1: 进区域关闭通信模块
        uint16_t innotcollectgnss;        // 0: 进区域不采集 GNSS 详细定位
                                          // 数据; 1: 进区域采集 GNSS 详细
                                          // 定位数据
    } bit;
    uint16_t value;
};

union RouteAttribute
{
    struct Bit
    {
        uint16_t bytime : 1;           // 1: 根据时间
        uint16_t retain1 : 1;          // 保留
        uint16_t inalarmtodirver : 1;  // 1: 进路线报警给驾驶员
        uint16_t inalarmtoserver : 1;  // 1: 进路线报警给平台
        uint16_t outalarmtodirver : 1; // 1: 出路线报警给驾驶员
        uint16_t outalarmtoserver : 1; // 1: 出路线报警给平台
        uint16_t retain2 : 10;         // 保留
    } bit;
    uint16_t value;
};

union RoadSectionAttribute
{
    struct Bit
    {
        uint8_t traveltime : 1;  // 1: 行驶时间
        uint8_t speedlimit : 1;  // 1: 限速
        uint8_t snlatitude : 1;  // 0: 北纬:1; 1: 南纬
        uint8_t ewlongitude : 1; // 0: 东经:1; 1: 西经
        uint8_t retain : 4;      // 保留
    } bit;
    uint8_t value;
};

struct Coordinate
{
    uint32_t latitude;  // 纬度
    uint32_t longitude; // 经度
};

struct InflectionPoint
{
    uint32_t inflection_point_id;                // 拐点ID
    uint32_t road_section_id;                    // 路段ID
    QPoint coordinate;                           // 拐点经纬度
    uint8_t road_section_wide;                   // 路段宽度, 路段为该拐点到下一拐点
    RoadSectionAttribute road_section_attribute; // 路段属性
    uint16_t max_driving_time;                   // 路段行驶过长阈值,
                                                 // 若路段属性 0 位为 0 则没有该字段
    uint16_t min_driving_time;                   // 路段行驶不足阈值,
                                                 // 若路段属性 0 位为 0 则没有该字段
    uint16_t max_speed;                          // 最高速度,
                                                 // 若路段属性 1 位为 0 则没有该字段
    uint8_t overspeed_duration;                  // 超速持续时间,
                                                 // 若路段属性 1 位为 0 则没有该字段
    uint8_t max_speed_night;                     // 夜间最高速度
    uint16_t name_len;                           // 区域名称长度
    std::string area_name;                       // 区域名称
};

class Cjt808Area
{
public:
    Cjt808Area() {}
    ~Cjt808Area() {}
    // 检查是否在区域内
    virtual bool isGpsContain(QPoint pt) = 0;
    // 检查是否处于限制时间段
    virtual bool isTimeContain();
    // 检查是否超速(白天/晚上)
    virtual bool isOverSpeed();
    // 检查是否超速时长越限
    virtual bool isOverSpeedTimeExceed();
    // 是否已经上报
    virtual bool isReported();
public:
    uint32_t area_id;             // 区域ID
    AreaAttribute area_attribute; // 区域属性
    uint8_t start_time[6];        // 起始时间
    uint8_t end_time[6];          // 结束时间
    uint16_t max_speed;           // 最高速度
    uint8_t overspeed_duration;   // 超速持续时间
    uint8_t max_speed_night;      // 夜间最高速度
    uint16_t name_len;            // 区域名称长度
    std::string area_name;        // 区域名称

protected:
    int overspeed_cnt; // 超速持续时间累计
};

// 圆形区域属性
class CircularArea : public Cjt808Area
{
public:
    CircularArea() {}
    ~CircularArea() {}

    // 检查是否在区域内
    virtual bool isGpsContain(QPoint pt);

public:
    QPoint center_point; // 中心点坐标
    uint32_t radius;     // 半径
};

// 矩形区域属性
class RectangleArea : public Cjt808Area
{
public:
    RectangleArea() {}
    ~RectangleArea() {}

    // 检查是否在区域内
    virtual bool isGpsContain(QPoint pt);

public:
    QPoint upper_left_corner;   // 左上点坐标
    QPoint bottom_right_corner; // 右下点坐标
};

class PolygonalArea : public Cjt808Area
{
public:
    PolygonalArea() {}
    ~PolygonalArea() {}

    // 检查是否在区域内
    virtual bool isGpsContain(QPoint pt);

public:
    uint16_t coordinate_count; // 坐标总数
    QPolygon polygon;          // 坐标列表
};

class Route
{
public:
    Route() {}
    ~Route() {}

public:
    uint32_t route_id;                                  // 路线
    RouteAttribute route_attribute;                     // 路线属性
    uint8_t start_time[6];                              // 起始时间
    uint8_t end_time[6];                                // 结束时间
    uint16_t inflection_point_count;                    // 路线总拐点数
    std::vector<InflectionPoint> inflection_point_list; // 路线拐点列表
    uint16_t name_len;                                  // 区域名称长度
    std::string route_name;                             // 区域名称
};

#pragma pack(pop)

typedef QMap<DWORD, CircularArea> CYCLE_AREA_MAP;
typedef QMap<DWORD, RectangleArea> RECT_AREA_MAP;
typedef QMap<DWORD, PolygonalArea> POLYGON_AREA_MAP;
typedef QMap<DWORD, Route> ROUTE_MAP;

class CJt808AreaRoute
{
public:
    CJt808AreaRoute();
    virtual ~CJt808AreaRoute();

    /*******************************************
    函数功能: 电子围栏检查
    入口参数:
    出口参数:
    备    注:
    *******************************************/
    void eFenceCheck();

protected:
    /*******************************************
    函数功能: 发送通用应答
    入口参数: cycle_num 流水号
             id  消息ID
             result 执行结果
    出口参数:
    备    注:
    *******************************************/
    virtual WORD sendCommonAnswer(WORD cycle_num, WORD id, BYTE result) = 0;

    /*******************************************
    函数功能: 发送数据到JT08服务器
    入口参数: WORD cmd,       命令字
            BYTE *payload,  净荷
            WORD lenth      净荷长度
    出口参数:
    备    注:
    *******************************************/
    virtual WORD sendPayload2Jt808Serv(WORD cmd, BYTE *payload, WORD lenth) = 0;

    /*******************************************
    函数功能: 设置圆形区域
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    void procSetAreaRound(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 删除圆形区域
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    void procDelAreaRound(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 设置矩形区域
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    void procSetAreaRect(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 删除矩形区域
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    void procDelAreaRect(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 设置多边形区域
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    void procSetAreaPolygon(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 删除多边形区域
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    void procDelAreaPolygon(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 设置路线
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    void procSetAreaRoute(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 删除线路
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    void procDleAreaRoute(int cycle_num, BYTE *payload, WORD lenth);

    /*******************************************
    函数功能: 查询线路或区域信息
    入口参数: data 数据包
             lenth 长度
    出口参数:
    备    注:
    *******************************************/
    void procQueryAreaInfo(int cycle_num, BYTE *payload, WORD lenth);

protected:
    CYCLE_AREA_MAP m_mapCycleArea;
    RECT_AREA_MAP m_mapRectArea;
    POLYGON_AREA_MAP m_mapPolygonArea;
    ROUTE_MAP m_mapRoute;
};

#endif // CJT808ALARM_H
