#include "CLoger.h"
#include <QDateTime>
#include <QDebug>

#define LOG_CONTENT_SIZE 1024

CLoger::CLoger()
    :m_maxRecordCnt(10000)
{
}

CLoger::CLoger(QString path, QString log_name, int max_count)
    : m_bBackup(false),
      m_iShowLevel(LOG_LEVEL_ERROR),
      m_iRecLevel(LOG_LEVEL_TRACE),
      m_maxRecordCnt(max_count)

{
    m_strLogLevel[LOG_LEVEL_FATAL] = "FATAL";
    m_strLogLevel[LOG_LEVEL_ERROR] = "ERROR";
    m_strLogLevel[LOG_LEVEL_WARN] = "WARNING";
    m_strLogLevel[LOG_LEVEL_INFO] = "INFO";
    m_strLogLevel[LOG_LEVEL_DEBUG] = "DEBUG";
    m_strLogLevel[LOG_LEVEL_TRACE] = "TRACE";

    if (path.at(path.length() - 1) != "/")
    {
        path += "/";
    }

    m_fullLog.setFileName(path + log_name + ".log");
    m_incLog.setFileName(path + log_name + ".inc");
    m_tmpLog.setFileName(path + log_name + ".tmp");

    // 增量日志文件
    m_incLog.open(QFile::ReadWrite | QFile::Append | QFile::Text);
}

bool CLoger::initialize()
{
    restore();
    return true;
}

bool CLoger::backup()
{
    if (m_bBackup)
    {
        return false;
    }

    m_bBackup = true;
    if (m_fullLog.isOpen())
    {
        m_fullLog.close();
    }

    // 全量持久化日志文件
    m_fullLog.open(QIODevice::WriteOnly | QIODevice::Text);
    foreach (QString item, m_LogInfo)
    {
        std::string info = item.toStdString();
        m_fullLog.write(info.c_str(), info.length());
    }

    // 将临时队列日志加入到当前队列
    {
        QMutexLocker locker(&m_mtxInc);
        foreach (auto item, m_incLogTemp)
        {
            std::string info = item.toStdString();
            m_fullLog.write(info.c_str(), info.length());
            m_LogInfo.push_back(item);
        }

        m_incLogTemp.clear();

        m_incLog.remove();
    }

    m_fullLog.close();
    m_bBackup = false;
    return true;
}

//<2021-10-21 10:59:30>[WARNING]<xxxx.cpp 2013> xxxxxxxxxx
bool getLogDate(QString str, QDate &date)
{
    int idx = str.indexOf(' ');
    if (idx == -1)
    {
        return false;
    }

    QString str_date = str.mid(1, idx -1);

    date= QDate::fromString(str_date, "yyyy-MM-dd");
    return true;
}

bool CLoger::restore()
{
    m_LogInfo.clear();
    // 取当前日期
    QDate date = QDateTime::currentDateTime().date();

    // 全量持久化日志文件
    if (m_fullLog.open(QIODevice::ReadWrite | QIODevice::Text))
    {
        QDate date1;
        while (!m_fullLog.atEnd())
        {
            QByteArray line = m_fullLog.readLine();
            QString str(line);
            // 获取日志中解析时间
            getLogDate(str, date1);
            // 取30天内的有效日志记录
            int days = date1.daysTo(date);
            if ( days < 30)
            {
                // 将数据写入日志循环队列
                m_LogInfo.append(str);
            }
        }
        m_fullLog.close();
    }

    {
        QMutexLocker locker(&m_mtxInc);
        // 全量持久化日志文件
        if (m_incLog.isOpen())
        {
            m_incLog.close();
        }

        if (m_incLog.open(QIODevice::ReadWrite | QIODevice::Text))
        {
            QDate date1;
            while (!m_incLog.atEnd())
            {
                QByteArray line = m_incLog.readLine();
                QString str(line);
                // 获取日志中解析时间
                getLogDate(str, date1);
                // 取30天内的有效日志记录
                int days = date.daysTo(date1);
                if ( days < 30)
                {
                    // 将数据写入日志循环队列
                    m_LogInfo.append(str);
                }
            }

            m_incLog.close();
        }
    }

    int total_cnt = m_LogInfo.size();
    if (total_cnt > m_maxRecordCnt)
    {
        m_LogInfo = m_LogInfo.mid(total_cnt - m_maxRecordCnt);
    }

    backup();
    return true;
}

void CLoger::setShowLevel(LOG_LEVEL level)
{
    m_iShowLevel = level;
}

void CLoger::setRecLevel(LOG_LEVEL level)
{
    m_iRecLevel = level;
}

void CLoger::logWrite(LOG_LEVEL level, const char *fmt, ...)
{
    va_list argptr;
    va_start(argptr, fmt);
    writeLogInfo(level, fmt, argptr);
    va_end(argptr);
}

void CLoger::writeLogInfo(LOG_LEVEL level, const char *fmt, va_list arg_list)
{
    if (level < LOG_LEVEL_FATAL || level > LOG_LEVEL_TRACE)
    {
        return;
    }

    char log_info[LOG_CONTENT_SIZE] = {0};
    vsnprintf(log_info, LOG_CONTENT_SIZE, fmt, arg_list);
    if (level <= m_iShowLevel)
    {
        qDebug() << log_info;
    }

    QMutexLocker locker(&m_mtxInc);
    QString strRecord = QString("<%1>[%2]:%3\r\n").arg(QDateTime::currentDateTime().toString("yyyy-MM-dd hh-mm-ss")).arg(m_strLogLevel[level]).arg(log_info);

    if (level <= m_iRecLevel)
    {
        if (m_bBackup)
        {
            m_incLogTemp.push_back(strRecord);
        }
        else
        {
            m_LogInfo.push_back(strRecord);
            if (!m_incLog.isOpen())
            {
                m_incLog.open(QFile::ReadWrite | QFile::Append | QFile::Text);
            }

            m_incLog.write(strRecord.toStdString().c_str(), strRecord.toStdString().length());
            m_incLog.flush();
        }
    }
}
