#include "CWebSocketClient.h"
#include "../socket/TcpServer.h"
#include "../base_module/factory.h"
#include "device/CDeviceManager.h"
#include "camera/CCameraManager.h"
#include "file/CFileManager.h"
#include <QMetaType>
#include <QDebug>
#include "assert.h"

REGISTER_MODULE(CWebSocketClient, "CWebSocketClient")

LOG_DEFINITION(CWebSocketClient)

// ws接口定义
const char *WS_LOGIN_COMMAND = "ws://%1";

CWebSocketClient::CWebSocketClient(QObject *parent)
    : QObject(parent),
      IModule(),
      m_pTcpModule(nullptr),
      m_pWebSock(nullptr),
      m_pFileMng(nullptr),
      m_pDevMng(nullptr),
      m_pCamMng(nullptr),
      m_bReady(false)
{
    m_pWebSock = new QWebSocket();
    connect(m_pWebSock, &QWebSocket::connected, this, &CWebSocketClient::onWebSockConnected);
    connect(m_pWebSock, &QWebSocket::textMessageReceived, this, &CWebSocketClient::onTextMessageReceived);
    connect(m_pWebSock, &QWebSocket::disconnected, this, &CWebSocketClient::closeConnection);
    connect(m_pWebSock, SIGNAL(stateChanged(QAbstractSocket::SocketState)), this, SLOT(onStateChanged(QAbstractSocket::SocketState)), Qt::DirectConnection);

    connect(this, SIGNAL(notifyConnect2Sock(QString)), this, SLOT(onConnect2Sock(QString)));

    connect(this, SIGNAL(notifySendTextMessage(QString)), this, SLOT(sendTextMessage(QString)));

    // 登陆响应
    m_pFuncList["onLogin"] = &CWebSocketClient::onLogin;
    // 获取参数响应
    m_pFuncList["onGetParams"] = &CWebSocketClient::onGetParams;
    // 更新系统
    m_pFuncList["upgradeSystem"] = &CWebSocketClient::upgradeSystem;
    // 回复出厂值
    m_pFuncList["restorefactorysettings"] = &CWebSocketClient::restoreFactorySettings;
    // 删除文件
    m_pFuncList["fileDelete"] = &CWebSocketClient::fileDelete;
    // 实时视频rtmp推送
    m_pFuncList["realtimeVideo"] = &CWebSocketClient::realtimeVideo;
    // 停止实时视频推流
    m_pFuncList["stopRealtimeVideo"] = &CWebSocketClient::stopRealtimeVideo;
    // 服务器要求获取录像文件的文件列表
    m_pFuncList["videoRecordList"] = &CWebSocketClient::videoRecordList;
    // 抓拍
    m_pFuncList["snapShot"] = &CWebSocketClient::snapShot;

    LOG_CREATE(CWebSocketClient, "web_sock");
}

CWebSocketClient::~CWebSocketClient()
{
    std::cout << name() << std::endl;
    SAFE_DELETE(m_pWebSock);
}

//初始化操作
bool CWebSocketClient::Initialize()
{
    qRegisterMetaType<QAbstractSocket::SocketState>("QAbstractSocket::SocketState");
    //起主消息处理线程
    STARTUP_TASK(TaskMain)

    return true;
}

// 是否准备就绪
bool CWebSocketClient::isReady()
{
    return m_bReady;
}

// 状态变更处理，这里只需要打桩即可
void CWebSocketClient::onStateChanged(QAbstractSocket::SocketState state)
{
    switch (state)
    {
    case QAbstractSocket::ConnectedState:
        LOG_WARN(CWebSocketClient, "QAbstractSocket::ConnectedState");
        break;
    case QAbstractSocket::ConnectingState:
        LOG_WARN(CWebSocketClient, "QAbstractSocket::ConnectingState");
        break;
    case QAbstractSocket::ClosingState:
        LOG_WARN(CWebSocketClient, "QAbstractSocket::ClosingState");
        break;
    case QAbstractSocket::UnconnectedState:
        LOG_WARN(CWebSocketClient, "QAbstractSocket::UnconnectedState");
        break;
    default:
        break;
    }
}

// 注册模块消息
bool CWebSocketClient::RegistMessage()
{
    m_pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
    if (m_pTcpModule)
    {
        m_pTcpModule->RegistMessageCallback(this, MODULE_WEB_SOCKET_MNG);
    }
    return true;
}

//主消息任务，主要用于和其他模块之间的消息通讯
void CWebSocketClient::TaskMain(void *pBuff)
{
    (void)pBuff;
    setCurrentTaskName("tskWebSock");
    MODULE_MESSAGE *pMsg = nullptr;
    RegistMessage();
    waitForStart();

    m_pDevMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
    m_pCamMng = dynamic_cast<CCameraManager *>(QueryPlugin("CCameraManager"));
    m_pFileMng = dynamic_cast<CFileManager *>(QueryPlugin("CFileManager"));

    while (!isShutdown())
    {
        if (GetModuleMessage(pMsg)) // get msg from message queue
        {
            switch (pMsg->iCmd)
            {
            //  case CMD_WEBSOCKER_CONNECT_TO_SERV:
            //      procConnect2Serv(pMsg);
            //      break;
            //  case CMD_SEND_TEXT_TO_SERV:
            //      procSendTextMessage(pMsg);
            //      break;
            default:
                break;
            }

            SAFE_DELETE(pMsg);
        }
    }
}

// 和业软服务端建立链接
void CWebSocketClient::connectToServer(QString strServAddr)
{
    emit notifyConnect2Sock(strServAddr);
}

// 连接处理操作
void CWebSocketClient::onConnect2Sock(QString strServAddr)
{
    if (strServAddr == m_strServIpPort && m_bReady == true)
    {
        emit notifyServConnected();
        return;
    }

    if (!strServAddr.isEmpty())
    {
        m_strServIpPort = strServAddr;
    }

    if (m_strServIpPort.indexOf("127.0.0.1") >= 0)
    {
        m_strServIpPort = m_strServIpPort.left(strServAddr.lastIndexOf('/') + 1);
    }
    QUrl url = QUrl(m_strServIpPort);
    m_pWebSock->open(url);
}

// 建立连接后处理。
void CWebSocketClient::onWebSockConnected()
{
    LOG_WARN(CWebSocketClient, "CWebSocketClient::onWebSockConnected!");
    if (m_pDevMng)
    {
        m_pDevMng->setNetworkStatus(false);
    }

    m_bReady = true;
    emit notifyServConnected();
}

// 关闭连接
void CWebSocketClient::closeConnection()
{
    LOG_WARN(CWebSocketClient, "CWebSocketClient::closeConnection!");

    if (m_pDevMng)
    {
        m_pDevMng->setNetworkStatus(true);
    }
    m_pWebSock->close();
    m_bReady = false;
}

// 接收到文本消息后的处理
void CWebSocketClient::onTextMessageReceived(QString message)
{
    QString strCmd, strJson;
    QVariantMap prop_arr;
    QString info = message.replace("\n", "");
    info = info.replace("    ", "");
    LOG_INFO(CWebSocketClient, "[web msg recv:%s]", info.toStdString().c_str());

    if (nullptr == m_pDevMng)
    {
        m_pDevMng = dynamic_cast<CDeviceManager *>(QueryPlugin("CDeviceManager"));
    }

    int idx = message.indexOf('{');
    if (idx >= 0)
    {
        // QString test_str = message.left(idx);
        // strCmd = test_str.mid(test_str.indexOf(':')+2);

        strJson = message.mid(idx);
        // 1. 创建 QJsonParseError 对象，用来获取解析结果
        QJsonParseError error;
        // 2. 使用静态函数获取 QJsonDocument 对象
        QJsonDocument jsonDocument = QJsonDocument::fromJson(strJson.toUtf8(), &error);
        // 3. 根据解析结果进行处理
        if (error.error == QJsonParseError::NoError)
        {
            if (!(jsonDocument.isNull() || jsonDocument.isEmpty()))
            {
                if (jsonDocument.isObject())
                {
                    prop_arr = QJsonObject(jsonDocument.object()).toVariantMap();
                }
                else if (jsonDocument.isArray())
                {
                    // ...
                    prop_arr = QJsonObject(jsonDocument.object()).toVariantMap();
                    ;
                }
            }
        }
        else
        {
            // 检查错误类型
            info = strJson.replace("\n", "");
            info = info.replace("    ", "");
            LOG_ERROR(CWebSocketClient, "[Parse json:%s, error:%s]", info.toUtf8().data(), error.errorString().toStdString().c_str());
        }
    }

    // 根据消息内容分发消息到具体模块
    onPraseServCmd(prop_arr);
}

// 发送消息到主机
void CWebSocketClient::sendTextMessage(QString msg)
{
    if (!m_bReady)
    {
        emit notifyConnect2Sock("");
    }

    qint64 ret = m_pWebSock->sendTextMessage(msg);
    QString info = msg.replace("\n", "");
    info = info.replace("    ", "");
    LOG_INFO(CWebSocketClient, "[sendTextMessage msg:%s, result:%d]", info.toStdString().c_str(), (int)ret);
}

// 发送消息到主机
bool CWebSocketClient::sendMessage(QVariant cmd)
{
    if (cmd != "")
    {
        QJsonDocument jsonDocument = QJsonDocument::fromVariant(cmd);
        if (!jsonDocument.isNull())
        {
            QString text = jsonDocument.toJson();
            emit notifySendTextMessage(text);
        }
    }
    return true;
}

// 解析服务器下发命令或响应
bool CWebSocketClient::onPraseServCmd(QVariantMap &prop_arr)
{
    qDebug() << "prop_arr" << prop_arr;
    QVariantMap::Iterator itCmd = prop_arr.find("msgName");
    if (itCmd != prop_arr.end())
    {
        QString strCmd = itCmd.value().toString();

        MAP_FUNC_PROC::Iterator it = m_pFuncList.find(strCmd);
        if (it != m_pFuncList.end())
        {
            WS_COMMAND_PROC_FUNC func = it.value();
            (this->*func)(prop_arr);
        }
        return true;
    }

    return false;
}

// 登陆响应
bool CWebSocketClient::onLogin(QVariantMap &prop_arr)
{
    if (m_pDevMng)
    {
        m_pDevMng->onLogin(prop_arr);
    }

    return true;
}

// 获取参数
bool CWebSocketClient::onGetParams(QVariantMap &prop_arr)
{
    if (m_pDevMng)
    {
        m_pDevMng->onGetParams(prop_arr);
    }
    return true;
}

// 更新系统
bool CWebSocketClient::upgradeSystem(QVariantMap &prop_arr)
{
    if (m_pDevMng)
    {
        m_pDevMng->upgradeSystem(prop_arr);
    }
    return true;
}

// 回复出厂值
bool CWebSocketClient::restoreFactorySettings(QVariantMap &prop_arr)
{
    // type	number	是否清除数据。1表示清除数据并恢复出厂设置； 2表示不清除数据，只恢复出厂设置； 3表示只清除数据，不恢复出厂设置
    QVariantMap::Iterator it = prop_arr.find("type");
    if (it != prop_arr.end())
    {
        int type = it.value().toInt();
        MODULE_MESSAGE msg;
        msg.dst_type = APP_TYPE_MAIN;
        msg.dst_id = MODULE_CONFIG_MNG;
        msg.iCmd = CMD_CONFIG_RESTORE_FACTORY;
        int offset = 0;
        memcpy(msg.szBuff + offset, &type, sizeof(int));
        offset += sizeof(int);

        ITcpModule *pTcpModule = dynamic_cast<ITcpModule *>(QueryPlugin("TcpModule"));
        if (pTcpModule)
        {
            pTcpModule->SendMsg(msg, sizeof(msg));
        }
    }
    return true;
}

// 删除文件
bool CWebSocketClient::fileDelete(QVariantMap &prop_arr)
{
    if (m_pFileMng)
    {
        m_pFileMng->fileDelete(prop_arr);
    }
    return true;
}

// 实时视频rtmp推送
bool CWebSocketClient::realtimeVideo(QVariantMap &prop_arr)
{
    if (m_pCamMng)
    {
        m_pCamMng->realtimeVideo(prop_arr);
    }
    return true;
}

// 停止实时视频推流
bool CWebSocketClient::stopRealtimeVideo(QVariantMap &prop_arr)
{
    if (m_pCamMng)
    {
        m_pCamMng->stopRealtimeVideo(prop_arr);
    }
    return true;
}

// 服务器要求获取录像文件的文件列表
bool CWebSocketClient::videoRecordList(QVariantMap &prop_arr)
{
    if (m_pFileMng)
    {
        m_pFileMng->getVideoRecordList(prop_arr);
    }
    return true;
}

// 抓拍
bool CWebSocketClient::snapShot(QVariantMap &prop_arr)
{
    if (m_pCamMng)
    {
        m_pCamMng->snapShot(prop_arr);
    }
    return true;
}
