#ifndef CSWDLSTATE_H
#define CSWDLSTATE_H
#include "common.h"
#include <QObject>
#include <QMutex>
#include <QSemaphore>
#include <QMap>

class FileDownloader;

#define INSTANCE(class_name, instance) \
    static class_name *getInstace()    \
    {                                  \
        if (nullptr == instance)       \
        {                              \
            instance = new class_name; \
        }                              \
        return instance;               \
    }

// 状态机状态字
class State : public QObject
{
    Q_OBJECT
public:
    State()
        : m_pNextState(nullptr),
          m_pRollBackState(nullptr)
    {
    }
    virtual ~State()
    {
        SAFE_DELETE(m_pNextState)
        SAFE_DELETE(m_pRollBackState)
    }

    virtual void start() {}
    virtual void process() { return next(); }

    virtual void next() { emit notifyNext(m_pNextState); }
    virtual void rollback() { emit notifyRollback(m_pRollBackState); }
    void setNextState(State *pNextState) { m_pNextState = pNextState; }
    void setRollbackState(State *pRollBackState) { m_pRollBackState = pRollBackState; }

signals:
    //
    void notifyNext(State *);
    //
    void notifyRollback(State *);

protected:
    State *m_pNextState;
    State *m_pRollBackState;
};

// 空闲态
class IdelState : public State
{
    Q_OBJECT
public:
    IdelState() {}
    // 获取实例
    INSTANCE(IdelState, m_instance)

    virtual void start();
    // 处理过程
    virtual void process();
    //
    virtual void rollback() {}

private:
    static IdelState *m_instance;
};

// 下载结果
enum DOWNLOAD_RESULT
{
    DL_RESULT_FAILED = -1,
    DL_RESULT_NONE,
    DL_RESULT_SUCCESS,
};

// 节点操作类型
enum PACKAGE_OP_TYPE
{
    // 新增文件
    OP_TYPE_ADD,
    // 升级文件
    OP_TYPE_UPDATE,
    // 删除文件
    OP_TYPE_REMOVE,
    // 移动文件
    OP_TYPE_MOVE,
    // 生成目录
    OP_TYPE_MKDIR,
};

// 软件包节点信息
struct UPDATE_PACKAGE_NODE
{
    // 构造初始化
    UPDATE_PACKAGE_NODE()
        : pDloader(nullptr),
          result(DL_RESULT_SUCCESS),
          op_type(OP_TYPE_ADD)
    {
    }
    // 析构
    ~UPDATE_PACKAGE_NODE(){
        SAFE_DELETE(pDloader)}

    FileDownloader *pDloader;
    DOWNLOAD_RESULT result;
    PACKAGE_OP_TYPE op_type;
    QString strUrl;
    QString strLocation;
    QString strDstPath;
    QString strMovePath;
};

typedef QMap<QString, UPDATE_PACKAGE_NODE> PKG_NODE_MAP;

// 下载态
class DownloadState : public State
{
    Q_OBJECT
public:
    // 构造函数
    DownloadState();
    // 析构函数
    ~DownloadState();

    // 获取实例
    INSTANCE(DownloadState, m_instance)
    // 处理过程
    virtual void process();
    //
    virtual void next();
    //
    void setPackageUrl(QString strUrl) { m_strPkgUrl = strUrl; }

    void setBakPath(QString strUrl);
protected slots:
    void onPackageDloadFinished(QString strPath, bool bSuccess);
    // void onFileDloadFinished(QString strUrl, bool bSuccess);

protected:
    // void downloadFiles();

    bool onParsePkgScript();

    // // 解压升级包
    // bool downloadPackage();
    // 执行升级脚本
    bool execScript();

    void clearDloadMap();

private:
    static DownloadState *m_instance;

    FileDownloader *m_pDownloader;
    QString m_strPkgUrl;
    DOWNLOAD_RESULT m_dlResult;
    bool m_bScriptDloadSuccess;

    QSemaphore m_dlWaitSem;
    PKG_NODE_MAP m_mapPkgNode;
};

// 交换态
class SwapState : public State
{
    Q_OBJECT
public:
    SwapState() {}
    // 获取实例
    INSTANCE(SwapState, m_instance)
    // 处理过程
    virtual void process();
    virtual void next();

private:
    static SwapState *m_instance;
};

// 回滚态
class RollbackState : public State
{
    Q_OBJECT
public:
    RollbackState() {}
    // 获取实例
    INSTANCE(RollbackState, m_instance)
    // 处理过程
    virtual void process();

private:
    static RollbackState *m_instance;
};

#endif // CSWDLSTATE_H
